from . import utils  # noqa
from . import installers  # noqa
from . import docbuilders  # noqa
from . import filepath  # noqa
from . import config  # noqa
from . import cssbuildbaseplugin  # noqa
from . import sassbuild  # noqa
from . import lessbuild  # noqa
from . import coffeebuild  # noqa
from . import mediacopy  # noqa
from . import bowerinstall  # noqa
from . import npminstall  # noqa
from . import browserify_jsbuild  # noqa
from . import browserify_babelbuild  # noqa
from . import browserify_reactjsbuild  # noqa
from . import typescriptbuild  # noqa
from . import nodemodulescopy  #noqa
from . import run_jstests  #noqa
from . import autosetup_jsdoc  #noqa
from . import autosetup_esdoc  #noqa
from . import npmrun_jsbuild  # noqa
from . import npmrun  # noqa
from . import single_media_copy

__all__ = [
    'utils',
    'installers',
    'docbuilders',
    'filepath',
    'config',
    'cssbuildbaseplugin',
    'sassbuild',
    'lessbuild',
    'coffeebuild',
    'mediacopy',
    'bowerinstall',
    'npminstall',
    'browserify_jsbuild',
    'browserify_babelbuild',
    'browserify_reactjsbuild',
    'typescriptbuild',
    'nodemodulescopy',
    'run_jstests',
    'autosetup_jsdoc',
    'autosetup_esdoc',
    'single_media_copy',
]



from django.contrib import admin

from ievv_opensource.ievv_sms.models import DebugSmsMessage


@admin.register(DebugSmsMessage)
class DebugSmsMessageAdmin(admin.ModelAdmin):
    list_per_page = 20

    list_display = [
        'phone_number',
        'message',
        'created_datetime',
    ]

    readonly_fields = [
        'phone_number',
        'message',
        'created_datetime',
    ]

    search_fields = [
        'phone_number',
        'message',
    ]

    def has_add_permission(self, request):
        return False



from . import base  # noqa
from . import django_runserver  # noqa
from . import dbdev_runserver  # noqa
from . import ievv_buildstatic  # noqa
from . import redis_server  # noqa
from . import celery_worker  # noqa
from . import rq_worker  # noqa



import argparse
import logging
import os
import re
import sys
import textwrap

import sh

try:
    from shlex import quote as cmd_quote
except ImportError:
    from pipes import quote as cmd_quote


logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO,
                    format='[%(levelname)s] %(message)s')
logging.getLogger('sh.command').setLevel(logging.WARNING)


def _find_management_commands():
    commands = []
    try:
        pythoncommand = sh.Command('python')
    except sh.CommandNotFound:
        pass
    else:
        output = pythoncommand('manage.py', '--help')
        for line in output.split():
            line = line.strip()
            if line.startswith('ievvtasks_'):
                commands.append(line[len('ievvtasks_'):])
    return commands


def _make_cli_epilog(commands):
    cli_help = """
    Available commands:
      {}
    """.format('\n      '.join(commands))
    return cli_help


class UnknownArgsParser(object):
    short_option_pattern = re.compile(
        '^-(?P<option>[a-zA-Z0-9][a-zA-Z0-9_-]*)=(?P<value>.+)$')
    long_option_pattern = re.compile(
        '^--(?P<option>[a-zA-Z0-9][a-zA-Z0-9_-]*)=(?P<value>.+)$')

    def __init__(self, unknown_args):
        self.unknown_args = unknown_args
        self.fixed_unknown_args = self.__parse()

    def __parse_argument(self, argument):
        long_option_match = self.long_option_pattern.match(argument)
        short_option_match = self.short_option_pattern.match(argument)
        if long_option_match:
            groupdict = long_option_match.groupdict()
            return '--{option}={value}'.format(
                option=groupdict['option'],
                value=cmd_quote(groupdict['value']),
            )
        elif short_option_match:
            groupdict = short_option_match.groupdict()
            return '-{option}={value}'.format(
                option=groupdict['option'],
                value=cmd_quote(groupdict['value']),
            )
        else:
            return cmd_quote(argument)

    def __parse(self):
        fixed_unknown_args = []
        for argument in self.unknown_args:
            fixed_unknown_args.append(self.__parse_argument(argument))
        return fixed_unknown_args

    def resultlist(self):
        return self.fixed_unknown_args

    def resultstring(self):
        return ' '.join(self.fixed_unknown_args)


def cli():
    commands = _find_management_commands()
    commands = list(set(commands))
    commands.sort()

    args = sys.argv[1:]
    parser = argparse.ArgumentParser(
        description='IEVV command line interface.',
        epilog=textwrap.dedent(_make_cli_epilog(commands)),
        formatter_class=argparse.RawDescriptionHelpFormatter,
        # Do not include help unless we just run ``ievv --help`` or ``ievv``.
        # If we do not use this, we can not add help for the sub commands.
        add_help=len(args) <= 1)
    parser.add_argument('command', type=str,
                        metavar='command',
                        help='The command to run. Use ``ievv <command> --help`` for '
                             'help with a specific command. The available commands '
                             'is listed in the "Available commands" section below.',
                        choices=commands)
    if len(args) == 0:
        parser.print_help()
    else:
        args, unknown_args = parser.parse_known_args()

        # if args.command == 'createproject':
        #     # parser = argparse.ArgumentParser(
        #     #     description='Initialize a new project')
        #     # parser.add_argument('command', type=str)
        #     # args = parser.parse_args(unknown_args)
        # else:
        parsed_unknown_args = UnknownArgsParser(unknown_args)
        os.system('python manage.py ievvtasks_{} {}'.format(
            args.command, parsed_unknown_args.resultstring()))



from . import browserify_babelbuild


class Plugin(browserify_babelbuild.Plugin):
    """
    Browserify javascript babel build plugin.

    Examples:

        Simple example::

            IEVVTASKS_BUILDSTATIC_APPS = ievvbuildstatic.config.Apps(
                ievvbuildstatic.config.App(
                    appname='demoapp',
                    version='1.0.0',
                    plugins=[
                        ievvbuildstatic.browserify_reactjsbuild.Plugin(
                            sourcefile='app.js',
                            destinationfile='app.js',
                        ),
                    ]
                )
            )

        Custom source folder example::

            IEVVTASKS_BUILDSTATIC_APPS = ievvbuildstatic.config.Apps(
                ievvbuildstatic.config.App(
                    appname='demoapp',
                    version='1.0.0',
                    plugins=[
                        ievvbuildstatic.browserify_reactjsbuild.Plugin(
                            sourcefolder=os.path.join('scripts', 'javascript', 'api'),
                            sourcefile='api.js',
                            destinationfile='api.js',
                        ),
                    ]
                )
            )

    """
    name = 'browserify_reactjsbuild'

    def install(self):
        """
        Installs the ``babel-preset-react`` NPM package
        in addition to the packages installed by
        :meth:`ievv_opensource.utils.ievvbuildstatic.browserify_babelbuild.Plugin.install`.

        The packages are installed with no version specified, so you
        probably want to freeze the versions using the
        :class:`ievv_opensource.utils.ievvbuildstatic.npminstall.Plugin` plugin.
        """
        super(Plugin, self).install()
        self.app.get_installer('npm').queue_install(
            'babel-preset-react')

    def get_babel_presets(self):
        """
        Adds ``react`` to the babelify presets added by
        :meth:`ievv_opensource.utils.ievvbuildstatic.browserify_babelbuild.Plugin.get_babel_presets`.
        """
        return super(Plugin, self).get_babel_presets() + [
            'react',
        ]

    def get_watch_extensions(self):
        return super(Plugin, self).get_watch_extensions() + [
            '.jsx'
        ]



from django.http import HttpResponseRedirect
from django.utils import translation
from django.utils.deprecation import MiddlewareMixin

from ievv_opensource.ievv_i18n_url import active_i18n_url_translation

from . import i18n_url_utils


class LocaleMiddleware(MiddlewareMixin):
    """
    `ievv_js_i18n_url` locale middleware.
    """
    response_redirect_class = HttpResponseRedirect

    def process_request(self, request):
        """
        Initializes the ievv_i18n_url handler from the request,
        and calls :meth:`ievv_opensource.ievv_i18n_url.handlers.AbstractHandler.activate_languagecode_from_request`.

        Args:
            request: The request-object.
        """
        handler_class = i18n_url_utils.get_handler_class()
        handler_class.activate_languagecode_from_request(request=request)
        translation_language = translation.get_language()
        request.LANGUAGE_CODE = translation_language
        request.session['LANGUAGE_CODE'] = translation_language
        request.IEVV_I18N_URL_DEFAULT_LANGUAGE_CODE = active_i18n_url_translation.get_default_languagecode()
        request.IEVV_I18N_URL_ACTIVE_LANGUAGE_CODE = active_i18n_url_translation.get_active_languagecode()

    def process_response(self, request, response):
        language = translation.get_language()
        if 'Content-Language' not in response:
            response['Content-Language'] = language
        return response



import json

from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.db import models
from django.utils import timezone


class BatchOperationManager(models.Manager):
    """
    Manager for :class:`.BatchOperation`.
    """
    def __create(self, input_data, **kwargs):
        batchoperation = BatchOperation(**kwargs)
        if input_data:
            batchoperation.input_data = input_data
        batchoperation.clean()
        batchoperation.save()
        return batchoperation

    def create_synchronous(self, input_data=None, **kwargs):
        """
        Create a synchronous :class:`.BatchOperation`.

        An synchronous batch operation starts with :obj:`.BatchOperation.status` set
        to :obj:`.BatchOperation.STATUS_RUNNING` and ``started_running_datetime``
        set just as if :meth:`.BatchOperation.mark_as_running` was called. So
        calling this would have the same result as calling :meth:`.create_asynchronous`
        and then calling :meth:`.BatchOperation.mark_as_running`, but this will
        just use one database query instead of two.

        The :class:`.BatchOperation` is cleaned before it is saved.

        Args:
            input_data: The input data.
                A python object to set as the input data using the
                :meth:`.BatchOperation.input_data` property.
            **kwargs: Forwarded to the constructor for :class:`.BatchOperation`.

        Returns:
            BatchOperation: The created BatchOperation object.
        """
        return self.__create(input_data=input_data,
                             status=BatchOperation.STATUS_RUNNING,
                             started_running_datetime=timezone.now(),
                             **kwargs)

    def create_asynchronous(self, input_data=None, **kwargs):
        """
        Create an asynchronous :class:`.BatchOperation`. An asynchronous
        batch operation starts with :obj:`.BatchOperation.status` set
        to :obj:`.BatchOperation.STATUS_UNPROCESSED`.

        The :class:`.BatchOperation` is cleaned before it is saved.

        Args:
            input_data: The input data.
                A python object to set as the input data using the
                :meth:`.BatchOperation.input_data` property.
            **kwargs: Forwarded to the constructor for :class:`.BatchOperation`.

        Returns:
            BatchOperation: The created BatchOperation object.
        """
        return self.__create(input_data=input_data, **kwargs)


class BatchOperation(models.Model):
    """
    Defines a batch operation.
    """

    objects = BatchOperationManager()

    #: One of the possible values for :obj:`~.BatchOperation.status`.
    #: Defines the BatchOperation as uprocessed (not yet started).
    #: This only makes sense for background tasks. They will typically
    #: be created with the unprocessed status, and then set to
    #: :obj:`~.BatchOperation.STATUS_RUNNING` when the batching service
    #: starts running the operation.
    STATUS_UNPROCESSED = 'unprocessed'

    #: One of the possible values for :obj:`~.BatchOperation.status`.
    #: Defines the BatchOperation as running (in progress).
    STATUS_RUNNING = 'running'

    #: One of the possible values for :obj:`~.BatchOperation.status`.
    #: Defines the BatchOperation as finished.
    STATUS_FINISHED = 'finished'

    #: Allowed values for :obj:`~.BatchOperation.status`.
    #: Possible values are:
    #:
    #: - :obj:`~.BatchOperation.STATUS_UNPROCESSED`.
    #: - :obj:`~.BatchOperation.STATUS_RUNNING`.
    #: - :obj:`~.BatchOperation.STATUS_FINISHED`.
    STATUS_CHOICES = [
        (STATUS_UNPROCESSED, 'unprocessed'),
        (STATUS_RUNNING, 'running'),
        (STATUS_FINISHED, 'finished'),
    ]

    #: One of the possible values for :obj:`~.BatchOperation.result`.
    #: This is used when we have no result yet (the operation is not finished).
    RESULT_NOT_AVAILABLE = 'not-available'

    #: One of the possible values for :obj:`~.BatchOperation.result`.
    #: Defines the BatchOperation as failed. This is set if the operation
    #: could not be completed because of an error. Any details about the
    #: result of the operation can be stored in :obj:`~.BatchOperation.output_data_json`.
    RESULT_SUCCESSFUL = 'successful'

    #: One of the possible values for :obj:`~.BatchOperation.result`.
    #: Defines the BatchOperation as failed. This is set if the operation
    #: could not be completed because of an error. Any error message(s)
    #: should be stored in :obj:`~.BatchOperation.output_data_json`.
    RESULT_FAILED = 'failed'

    #: Allowed values for :obj:`~.BatchOperation.result`.
    #: Possible values are:
    #:
    #: - :obj:`~.BatchOperation.RESULT_NOT_AVAILABLE`.
    #: - :obj:`~.BatchOperation.RESULT_SUCCESSFUL`.
    #: - :obj:`~.BatchOperation.RESULT_FAILED`.
    RESULT_CHOICES = [
        (RESULT_NOT_AVAILABLE, 'not available yet (processing not finished)'),
        (RESULT_SUCCESSFUL, 'successful'),
        (RESULT_FAILED, 'failed'),
    ]

    #: The user that started this batch operation.
    #: Optional, but it is good metadata to add for debugging.
    started_by = models.ForeignKey(
        to=settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        null=True, blank=True)

    #: The datetime when this batch operation was created.
    #: Defaults to ``timezone.now()``.
    created_datetime = models.DateTimeField(
        default=timezone.now)

    #: The datetime when this batch operation started running.
    #: This is not the same as :obj:`~.BatchOperation.created_datetime`,
    #: this is the time when the operation started processing.
    started_running_datetime = models.DateTimeField(
        null=True, blank=True)

    #: The datetime when this batch operation was finished.
    finished_datetime = models.DateTimeField(
        null=True, blank=True)

    # #: Is this an asynchronous operation? Set this to ``True`` for
    # #: background tasks (such as Celery tasks).
    # asynchronous = models.BooleanField(
    #     default=False)

    #: The content type for :obj:`~.BatchOperation.context_object`.
    context_content_type = models.ForeignKey(
        to=ContentType,
        on_delete=models.CASCADE,
        null=True,
        blank=True)

    #: The id field for :obj:`~.BatchOperation.context_object`.
    context_object_id = models.PositiveIntegerField(
        null=True, blank=True)

    #: Generic foreign key that identifies the context this operation
    #: runs in. This is optional.
    context_object = GenericForeignKey('context_content_type', 'context_object_id')

    #: The type of operation. This is application specific - you
    #: typically use this if you allow multiple different batch operations
    #: on the same :obj:`~BatchOperation.context_object`.
    #: This is not required, and defaults to empty string.
    operationtype = models.CharField(
        max_length=255, db_index=True,
        null=False, blank=True, default='')

    #: The status of the operation.
    #: The allowed values for this field is documented in
    #: :obj:`~.BatchOperation.STATUS_CHOICES`.
    #: Defaults to :obj:`~.BatchOperation.STATUS_UNPROCESSED`.
    status = models.CharField(
        max_length=12,
        choices=STATUS_CHOICES,
        default=STATUS_UNPROCESSED)

    #: The result of the operation.
    #: The allowed values for this field is documented in
    #: :obj:`~.BatchOperation.RESULT_CHOICES`.
    #: Defaults to :obj:`~.BatchOperation.RESULT_NOT_AVAILABLE`.
    result = models.CharField(
        max_length=13,
        choices=RESULT_CHOICES,
        default=RESULT_NOT_AVAILABLE)

    #: Input data for the BatchOperation.
    #: You should not use this directly, use the :obj:`~.BatchOperation.input_data`
    #: property instead.
    input_data_json = models.TextField(
        null=False, blank=True, default='')

    #: Output data for the BatchOperation.
    #: You should not use this directly, use the :obj:`~.BatchOperation.output_data`
    #: property instead.
    output_data_json = models.TextField(
        null=False, blank=True, default='')

    class Meta:
        ordering = ['-created_datetime']

    @property
    def input_data(self):
        """
        Decode :obj:`.BatchOperation.input_data_json` and return the result.

        Return `None` if input_data_json is empty.
        """
        if self.input_data_json:
            if not hasattr(self, '_input_data'):
                # Store the decoded input_data to avoid re-decoding the json for
                # each access. We invalidate this cache in the setter.
                self._input_data = json.loads(self.input_data_json)
            return self._input_data
        else:
            return None

    @input_data.setter
    def input_data(self, input_data):
        """
        Set :obj:`.BatchOperation.input_data_json`. Encodes the given
        input_data using `json.dumps`.
        """
        self.input_data_json = json.dumps(input_data)
        if hasattr(self, '_input_data'):
            delattr(self, '_input_data')

    @property
    def output_data(self):
        """
        Decode :obj:`.BatchOperation.output_data_json` and return the result.

        Returns:
            object: `None` if output_data_json is empty, or the decoded json data
            if the ``output_data`` is not empty.
        """
        if self.output_data_json:
            if not hasattr(self, '_output_data'):
                # Store the decoded output_data to avoid re-decoding the json for
                # each access. We invalidate this cache in the setter.
                self._output_data = json.loads(self.output_data_json)
            return self._output_data
        else:
            return None

    @output_data.setter
    def output_data(self, output_data):
        """
        Set :obj:`.BatchOperation.output_data_json`. Encodes the given
        output_data using `json.dumps`.

        Returns:
            object: `None` if output_data_json is empty, or the decoded json data
            if the ``output_data`` is not empty.
        """
        self.output_data_json = json.dumps(output_data)
        if hasattr(self, '_output_data'):
            delattr(self, '_output_data')

    def mark_as_running(self):
        """
        Mark the batch operation as running.

        Sets the :obj:`.status` to :obj:`.STATUS_RUNNING`,
        :obj:`.started_running_datetime` to the current datetime,
        clean and save.
        """
        self.status = self.STATUS_RUNNING
        self.started_running_datetime = timezone.now()
        self.clean()
        self.save()

    def finish(self, failed=False, output_data=None):
        """
        Mark the bulk operation as finished.

        Sets :obj:`.result` as documented in the ``failed`` parameter below.
        Sets :obj:`.finished_datetime` to the current datetime.
        Sets :obj:`.output_data_json` as documented in the ``output_data``
        parameter below.

        Args:
            failed (boolean): Set this to ``False`` to set :obj:`.result` to
                :obj:`.RESULT_FAILED`. The default is ``True``, which means that
                :obj:`.result` is set to :obj:`.RESULT_SUCCESSFUL`
            output_data: The output data.
                A python object to set as the output data using the
                :meth:`.BatchOperation.output_data` property.
        """
        if failed:
            self.result = self.RESULT_FAILED
        else:
            self.result = self.RESULT_SUCCESSFUL
        if output_data:
            self.output_data = output_data
        self.status = self.STATUS_FINISHED
        self.finished_datetime = timezone.now()
        self.clean()
        self.save()

    def clean(self):
        if self.status == self.STATUS_FINISHED and self.result == self.RESULT_NOT_AVAILABLE:
            raise ValidationError({
                'result': 'Must be "successful" or "failed" when status is "finished".'
            })
        if self.status != self.STATUS_UNPROCESSED and self.started_running_datetime is None:
            raise ValidationError({
                'started_running_datetime': 'Can not be None when status is "running" or "finished".'
            })

    def __str__(self):
        return \
            '#{id}-{operationtype}({created_datetime})[{status}, {result}]' \
            ' - {context_content_type}#{context_object_id}'.format(
                id=self.id,
                operationtype=self.operationtype,
                created_datetime=self.created_datetime,
                status=self.status,
                result=self.result,
                context_content_type=self.context_content_type,
                context_object_id=self.context_object_id
            )



from ievv_opensource.utils.ievvbuildstatic import pluginbase
from ievv_opensource.utils.ievvbuildstatic.installers.npm import NpmInstaller
from ievv_opensource.utils.shellcommandmixin import ShellCommandMixin


class Plugin(pluginbase.Plugin, ShellCommandMixin):
    """
    NPM install plugin --- installs NPM packages.

    The packages are installed when ``ievv buildstatic`` starts up.

    Examples:

        Install uniq and momentjs packages::

            IEVVTASKS_BUILDSTATIC_APPS = ievvbuildstatic.config.Apps(
                ievvbuildstatic.config.App(
                    appname='demoapp',
                    version='1.0.0',
                    plugins=[
                        ievvbuildstatic.npminstall.Plugin(
                            packages={
                                'uniq': None,
                                'momentjs': None
                            }
                        ),
                    ]
                )
            )

        You can specify a version instead of ``None`` if you do not want
        to install the latest version.
    """
    name = 'npminstall'

    def __init__(self, packages, **kwargs):
        super(Plugin, self).__init__(**kwargs)
        self.packages = packages

    def install(self):
        for package, version in self.packages.items():
            self.app.get_installer('npm').queue_install(
                package, version=version)



import os

from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'Run development servers configured for this project.'

    def add_arguments(self, parser):
        parser.add_argument('-n', '--name', dest='name',
                            required=False,
                            choices=settings.IEVVTASKS_DEVRUN_RUNNABLES.keys(),
                            default='default')
        parser.add_argument('-q', '--disable-desktop-notifications', dest='enable_desktop_notifications',
                            required=False, action='store_false', default=True)

    def handle(self, *args, **options):
        name = options['name']
        if not options['enable_desktop_notifications']:
            os.environ.setdefault('IEVV_OPENSOURCE_ENABLE_DESKTOP_NOTIFICATIONS', 'false')
        settings.IEVVTASKS_DEVRUN_RUNNABLES[name].start()



from __future__ import print_function
from __future__ import unicode_literals

import psutil
import sh


try:
    from shlex import quote as shell_quote
except ImportError:
    from pipes import quote as shell_quote


class ShellCommandError(Exception):
    """
    Raised when :meth:`.LogMixin.run_shell_command` fails.
    """


class ShellCommandMixin(object):
    """
    Shell command mixin - for classes that need to run shell commands.

    Requires :class:`~ievv_opensource.utils.logmixin.LogMixin`.
    """
    def log_shell_command_stdout(self, line):
        """
        Called by :meth:`.run_shell_command` each time the shell
        command outputs anything to stdout.
        """
        self.get_logger().stdout(line.rstrip())

    def log_shell_command_stderr(self, line):
        """
        Called by :meth:`.run_shell_command` each time the shell
        command outputs anything to stderr.
        """
        self.get_logger().stderr(line.rstrip())

    def prettyformat_shell_command(self, executable, args=None, kwargs=None, _cwd=None):
        output = [executable]
        if args:
            output.extend([shell_quote(arg) for arg in args])
        if kwargs:
            for key, value in kwargs.items():
                if value:
                    if len(key) == 1:
                        prefix = '-'
                    else:
                        prefix = '--'
                    if value is True:
                        formatted_value = ''
                    else:
                        formatted_value = '={}'.format(value)
                    output.append('{prefix}{key}{formatted_value}'.format(
                        prefix=prefix,
                        key=key,
                        formatted_value=formatted_value
                    ))
        if _cwd:
            output.insert(0, '[CWD={}]'.format(_cwd))

        return ' '.join(output)

    def run_shell_command(self, executable, args=None, kwargs=None, _cwd=None,
                          _out=None, _err=None, _env=None):
        """
        Run a shell command.

        Parameters:
            executable: The name or path of the executable.
            args: List of arguments for the ``sh.Command`` object.
            kwargs: Dict of keyword arguments for the ``sh.Command`` object.

        Raises:
            ShellCommandError: When the command fails. See :class:`.ShellCommandError`.
        """
        self.get_logger().debug('Execute: {}'.format(self.prettyformat_shell_command(
            executable=executable, args=args, kwargs=kwargs, _cwd=_cwd)))
        command = sh.Command(executable)
        args = args or []
        kwargs = kwargs or {}
        if _cwd:
            kwargs['_cwd'] = _cwd
        if _env:
            kwargs['_env'] = _env

        _out = _out or self.log_shell_command_stdout
        _err = _err or self.log_shell_command_stderr

        try:
            return command(*args,
                           _out=_out,
                           _err=_err,
                           **kwargs)
        except sh.ErrorReturnCode:
            # We do not need to show any more errors here - they
            # have already been printed by the _out and _err handlers.
            raise ShellCommandError()

    def kill_process(self, pid):
        """
        Kill the system process with the given ``pid``, and all
        its child processes.

        .. warning::

            You should normally use :meth:`.terminate_process` instead of
            this method since that normally gives the process the chance to
            cleanup.

        Parameters:
            pid: The process ID of the process you want to kill.

        Returns:
            A list of all the killed processes.
        """
        process_ids = []
        try:
            process = psutil.Process(pid)
        except psutil.NoSuchProcess:
            pass
        else:
            for childprocess in process.children():
                process_ids.append(childprocess.pid)
                childprocess.kill()
            process_ids.append(process.pid)
            process.kill()
        return process_ids

    def terminate_process(self, pid):
        """
        Terminate the system process with the given ``pid``, and all
        its child processes.

        Parameters:
            pid: The process ID of the process you want to terminate.

        Returns:
            A list of all the terminated processes.
        """
        process_ids = []
        try:
            process = psutil.Process(pid)
        except psutil.NoSuchProcess:
            pass
        else:
            for childprocess in process.children():
                process_ids.append(childprocess.pid)
                childprocess.terminate()
            try:
                process.terminate()
            except psutil.NoSuchProcess:
                pass
            else:
                process_ids.append(process.pid)
        return process_ids



from django.contrib import admin
from ievv_opensource.ievv_tagframework.models import Tag, TaggedObject


class TagAdmin(admin.ModelAdmin):
    list_display = [
        'id', 'taglabel', 'tagtype'
    ]
    list_filter = [
        'tagtype'
    ]
    fields = [
        'taglabel',
        'tagtype',
    ]

admin.site.register(Tag, TagAdmin)


class TaggedObjectAdmin(admin.ModelAdmin):
    list_display = [
        'id', 'tag', 'content_object'
    ]
    list_filter = [
        'tag__tagtype'
    ]
    fields = [
        'tag',
    ]

admin.site.register(TaggedObject, TaggedObjectAdmin)



import re

from PyInquirer import prompt


class VersionValidationError(Exception):
    """"""
    pass


class SemanticVersioning(object):
    """
    Updates an existing version number using semantic versioning.
    The incoming version number is expected to be a string without ``alpha`` in the end.

    Example::
        SemanticVersioning(version_string='10.0.0', project_name='My project').start()

        The user will be prompted with 3 choices in the terminal::
            * Patch (10.0.1)
            * Minor (10.1.0)
            * Major (11.0.0)

        A confirmation is asked for the choice.
        If the confirmation is ``False``, the 3 choices are prompted again until the confirmation is ``True``, then
        the version number is updated and returned
    """
    def __init__(self, version_string, project_name):
        self.project_name = project_name
        self.current_version = self.__validate_version_number(version_string=version_string)
        self.current_version_dict = self.__version_list_as_dict_with_ints(
            version_list=self.__split_string(version_string=self.current_version))

    def start(self):
        """
        Method which is called after initialising of the class :class:`SemanticVersioning` to make the update of an
        existing version number to a new one.
        Makes it possible for users to regret their choice of version number.

        :return: str(updated version)
        """
        new_version = self.__set_new_semantic_version_number()
        confirm_version = prompt(self.__confirm_new_version(chosen_version=new_version))
        while confirm_version['confirmed'] is False:
            new_version = self.__set_new_semantic_version_number()
            confirm_version = prompt(self.__confirm_new_version(chosen_version=new_version))
        validated_new_version = self.__validate_version_number(version_string=new_version)
        return validated_new_version

    def __set_new_semantic_version_number(self):
        """
        Validates the incoming version number and performs the update of the version before it is returned.
        Prompts the user to chose on of the three possible versions.
        Updates the old version number to the new one based on the user's choice.

        :return: str(new version number)
        """
        version_answer = prompt(
            self.__release_type_choices()
        )

        possible_new_version_dict = self.__set_possible_new_version_dict_with_strings()

        if version_answer.get('new_version', '').startswith('Patch'):
            new_version_number = possible_new_version_dict["new_patch"]
        elif version_answer.get('new_version', '').startswith('Minor'):
            new_version_number = possible_new_version_dict["new_minor"]
        else:
            new_version_number = possible_new_version_dict["new_major"]

        return new_version_number

    def __validate_version_number(self, version_string):
        """
        Validate version number.

        :param version_string: str()
        :return: str(validated version number) or Raises an error
        """
        if re.match(r'^\d+.\d+.\d+$', version_string):
            return version_string
        raise VersionValidationError(f'The provided version {version_string} does not match the expected pattern')

    def __split_string(self, version_string):
        """
        Makes a list of the version number before update with major, minor and patch as elements.

        :param version_string: str(version number before update)
        :return: list(old version number)
        """
        return version_string.split('.')

    def __version_list_as_dict_with_ints(self, version_list):
        """
        Makes a list with the keys ``major``, ``minor`` and ``patch`` where the values are the corresponding number
        from the old version number.

        :param version_list: list(Old version number as list)
        :return: dict(old version number as dict)
        """
        version_as_dict = {'major': int(version_list[0]), 'minor': int(version_list[1]), 'patch': int(version_list[2])}
        return version_as_dict

    def __set_possible_new_version_dict_with_strings(self):
        """
        Updates version number to match the possible kind of releases (major, minor and patch)

        :return: dict(Possible new version numbers with values as strings)
        """
        new_versions = {
            'new_major': self.get_next_major(),
            'new_minor': self.get_next_minor(),
            'new_patch': self.get_next_patch()
        }
        return new_versions

    def get_next_patch(self):
        """
        Get the next patch version based on the current version. Patch version is incremented by 1.

        :return: New version as string.
        """
        return f'{self.current_version_dict["major"]}.{self.current_version_dict["minor"]}.{self.current_version_dict.get("patch") + 1}'

    def get_next_minor(self):
        """
        Get the minor version based on the current version. Minor version is incremented by 1

        :return: New version as string.
        """
        return f'{self.current_version_dict["major"]}.{self.current_version_dict.get("minor") + 1}.0'

    def get_next_major(self):
        """
        Get the major version based on the current version. Major version is incremented by 1.

        :return: New version as string.
        """
        return f'{self.current_version_dict.get("major") + 1}.0.0'

    def __release_type_choices(self):
        """
        Uses ``PyInquirer`` to present the choices nicely

        :return: list(Release type choices as prompted to the user))
        """
        return [
            {
                'type': 'list',
                'name': 'new_version',
                'message': f'Select a new version of {self.project_name}. Currently at {self.current_version}',
                'choices': [
                    f"Patch ({self.get_next_patch()})",
                    f"Minor ({self.get_next_minor()})",
                    f"Major ({self.get_next_major()})"
                ]
            },
        ]

    def __confirm_new_version(self, chosen_version):
        """
        Uses ``PyInquirer`` to prompt the user a confirmation of the release choice
        :param chosen_version: str()
        :return: list(Confirmation of new release choice promted to the user)
        """
        return [
            {
                'type': 'confirm',
                'name': 'confirmed',
                'message': f'You have chosen {chosen_version} as new version number. Is this correct?'

            }
        ]


# Used for testing script in terminal
if __name__ == '__main__':
    SemanticVersioning(version_string="198.2.1", project_name='Test project').start()



from django import test

from ievv_opensource.ievv_sms import sms_registry


class TestAbstractSmsBackend(test.TestCase):
    def test_get_sms_text_length(self):
        self.assertEqual(sms_registry.AbstractSmsBackend.get_sms_text_length('Åge{^'), 7)

    def test_get_max_length(self):
        self.assertEqual(sms_registry.AbstractSmsBackend.get_max_length(), 918)

    def test_get_part_count_simple(self):
        self.assertEqual(sms_registry.AbstractSmsBackend.get_part_count('Test'), 1)

    def test_get_part_count_exactly_fill_1(self):
        self.assertEqual(sms_registry.AbstractSmsBackend.get_part_count('x' * 160), 1)

    def test_get_part_count_6(self):
        self.assertEqual(sms_registry.AbstractSmsBackend.get_part_count('x' * 900), 6)



import logging
import posixpath
import urllib.parse

from django.conf import settings
from ievv_opensource.ievv_i18n_url.base_url import BaseUrl
from ievv_opensource.ievv_i18n_url.handlers.abstract_handler import \
    UrlTransformError

from . import abstract_handler


logger = logging.getLogger(__name__)


class UrlpathPrefixHandler(abstract_handler.AbstractHandler):
    """
    I18n url handler that matches languages based on a URL prefix.
    """

    #: URL path prefix before we look for the languagecode in the url.
    #: **Must not** start or end with ``/``.
    #:
    #: E.g.:
    #:
    #: - If your languages should be served at ``/<languagecode>``, this should be blank string.
    #: - If your languages should be served at ``/my/translations/<languagecode>``, this should be ``my/translations``.
    LANGUAGECODE_URLPATH_PREFIX = ''

    @classmethod
    def get_urlpath_prefix_for_languagecode(cls, base_url, languagecode):
        if languagecode == cls.detect_default_languagecode(base_url):
            return ''
        if cls.LANGUAGECODE_URLPATH_PREFIX:
            return f'{cls.LANGUAGECODE_URLPATH_PREFIX}/{languagecode}'
        return languagecode

    @classmethod
    def _get_full_urlpath_prefix(cls):
        return f'/{cls.LANGUAGECODE_URLPATH_PREFIX}/'

    @classmethod
    def _strip_languagecode_from_urlpath(cls, base_url, languagecode, path):
        if not cls.is_supported_languagecode(languagecode):
            return path
        if languagecode == cls.detect_default_languagecode(base_url):
            return path
        if cls.LANGUAGECODE_URLPATH_PREFIX:
            full_prefix = cls._get_full_urlpath_prefix()
            if not path.startswith(full_prefix):
                return path
            path_without_prefix = path[len(full_prefix):]
        else:
            path_without_prefix = path[1:]
        splitpath = path_without_prefix.split(posixpath.sep, 1)
        if not splitpath:
            return path
        languagecode_in_path = splitpath[0]
        if languagecode_in_path != languagecode:
            return path
        return f'/{splitpath[1]}'

    @classmethod
    def _detect_languagecode_from_urlpath(cls, path):
        if cls.LANGUAGECODE_URLPATH_PREFIX:
            full_prefix = cls._get_full_urlpath_prefix()
            if not path.startswith(full_prefix):
                return None
            path_without_prefix = path[len(full_prefix):]
        else:
            path_without_prefix = path[1:]
        splitpath = path_without_prefix.split(posixpath.sep, 1)
        if not splitpath:
            return None
        languagecode = splitpath[0]
        if cls.is_supported_languagecode(languagecode):
            return languagecode
        return None

    @classmethod
    def get_languagecode_from_url(cls, url):
        urlpath = urllib.parse.urlparse(url).path
        return cls._detect_languagecode_from_urlpath(urlpath) or settings.LANGUAGE_CODE

    @classmethod
    def _build_urlpath(cls, path, languagecode, base_url):
        prefix = cls.get_urlpath_prefix_for_languagecode(
            base_url=base_url, languagecode=languagecode)
        if prefix:
            full_path = f'/{prefix}{path}'
        else:
            full_path = path
        return full_path

    def build_urlpath(self, path, languagecode=None, base_url=None):
        real_languagecode = languagecode or self.active_languagecode
        return self.__class__._build_urlpath(
            path=path,
            languagecode=real_languagecode,
            base_url=base_url or self.active_base_url)

    def build_absolute_url(self, path, languagecode=None, base_url=None):
        base_url = base_url or self.active_base_url
        return base_url.build_absolute_url(
            self.build_urlpath(path=path, languagecode=languagecode, base_url=base_url))

    @classmethod
    def transform_url_to_languagecode(cls, url, languagecode, translate_path=True):
        from_languagecode = cls.get_languagecode_from_url(url)
        base_url = BaseUrl(url)
        path_without_languagecode = cls._strip_languagecode_from_urlpath(
            base_url=base_url,
            languagecode=from_languagecode,
            path=urllib.parse.urlparse(url).path)
        if translate_path:
            try:
                path_without_languagecode = cls.transform_urlpath_to_languagecode(
                    base_url=base_url,
                    path=path_without_languagecode,
                    from_languagecode=from_languagecode,
                    to_languagecode=languagecode)
            except UrlTransformError as e:
                logger.warning('Failed to translate %s. Error: %s', path_without_languagecode, e)
        return base_url.build_absolute_url(
            cls._build_urlpath(path=path_without_languagecode, languagecode=languagecode, base_url=base_url))



import sys

from ievv_opensource.utils.ievvdevrun.runnables import base


class RunnableThread(base.ShellCommandRunnableThread):
    """
    redis-server runnable thread.

    Examples:

        You can just add it to your Django development settings with::

            IEVVTASKS_DEVRUN_RUNNABLES = {
                'default': ievvdevrun.config.RunnableThreadList(
                    ievvdevrun.runnables.redis_server.RunnableThread()
                )
            }

        Or if you want Redis to run with your custom ``redis.conf`` file::

            IEVVTASKS_DEVRUN_RUNNABLES = {
                'default': ievvdevrun.config.RunnableThreadList(
                    ievvdevrun.runnables.redis_server.RunnableThread(config_path=/path/to/config/redis.conf)
                )
            }

        And you can make it not restart on crash with::

            IEVVTASKS_DEVRUN_RUNNABLES = {
                'default': ievvdevrun.config.RunnableThreadList(
                    ievvdevrun.runnables.redis_server.RunnableThread(
                        autorestart_on_crash=False)
                )
            }

    """
    default_autorestart_on_crash = True

    def __init__(self, port='6379', config_path=None):
        """
        Args:
            port: The port to run the Redis server on. Defaults to ``"6379"``.
            config_path: Path to the redis.conf file. Defaults to ``None``.
        """
        self.port = port
        self.config_path = config_path
        super(RunnableThread, self).__init__()

    def get_logger_name(self):
        return 'Redis server'

    def get_command_config(self):
        args = ['--port', str(self.port)]
        if self.config_path:
            args.insert(0, self.config_path)
        return {
            'executable': 'redis-server',
            'args': args
        }



# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-04-30 13:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DevelopEmail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_datetime', models.DateTimeField(auto_now_add=True)),
                ('subject', models.TextField(blank=True, default='')),
                ('from_email', models.TextField(blank=True, db_index=True, default='')),
                ('to_emails', models.TextField(blank=True, default='')),
                ('raw_message', models.TextField(blank=True, default='')),
            ],
            options={
                'ordering': ['-created_datetime'],
            },
        ),
    ]



from .batchregistry import Action  # noqa
from .batchregistry import ActionGroup  # noqa
from .batchregistry import ActionGroupResult  # noqa
from .batchregistry import ActionResult  # noqa
from .batchregistry import ActionGroupExecutionInfo  # noqa
from .batchregistry import ActionError  # noqa
from .batchregistry import action_factory  # noqa



from unittest import TestCase
from ievv_opensource.ievvtasks_common import cli


class TestUnknownArgsParser(TestCase):
    def test_nothing_to_fix(self):
        self.assertEqual(
            ['a', '--test1=10', '--test2', 'c'],
            cli.UnknownArgsParser(['a', '--test1=10', '--test2', 'c']).resultlist()
        )

    def test_handle_equals_multiword_long_option(self):
        self.assertEqual(
            ["--test='Hello Cruel World'"],
            cli.UnknownArgsParser(['--test=Hello Cruel World']).resultlist()
        )

    def test_handle_equals_multiword_short_option(self):
        self.assertEqual(
            ["-t='Hello Cruel World'"],
            cli.UnknownArgsParser(['-t=Hello Cruel World']).resultlist()
        )

    def test_handle_equals_multiword_no_option(self):
        self.assertEqual(
            ["'Hello Cruel World'"],
            cli.UnknownArgsParser(['Hello Cruel World']).resultlist()
        )

    def test_resultlist(self):
        self.assertEqual(
            [
                "simple",
                "'Hello Cruel World'",
                "--test='A Test'",
                "-t='Another Test'",
                "--othertest", "'Other Test'"
            ],
            cli.UnknownArgsParser([
                'simple',
                'Hello Cruel World',
                '--test=A Test',
                '-t=Another Test',
                '--othertest', 'Other Test',
            ]).resultlist()
        )

    def test_resultstring(self):
        self.assertEqual(
            "simple 'Hello Cruel World' --test='A Test' "
            "-t='Another Test' --othertest 'Other Test'",
            cli.UnknownArgsParser([
                'simple',
                'Hello Cruel World',
                '--test=A Test',
                '-t=Another Test',
                '--othertest', 'Other Test',
            ]).resultstring()
        )



import random


def generate_django_secret_key():
    length = 50
    characters = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
    return ''.join(
        [random.SystemRandom().choice(characters) for i in range(length)])



from django.conf import settings
from termcolor import colored


#: Red color constant for :func:`.ievv_colorize`.
COLOR_RED = 'red'

#: Blue color constant for :func:`.ievv_colorize`.
COLOR_BLUE = 'blue'

#: Yellow color constant for :func:`.ievv_colorize`.
COLOR_YELLOW = 'yellow'

#: Grey color constant for :func:`.ievv_colorize`.
COLOR_GREY = 'grey'

#: Green color constant for :func:`.ievv_colorize`.
COLOR_GREEN = 'green'


def colorize(text, color, bold=False):
    """
    Colorize a string for stdout/stderr.

    Colors are only applied if :setting:`IEVV_COLORIZE_USE_COLORS` is
    ``True`` or not defined (so it defaults to ``True``).

    Examples:

        Print blue text::

            from ievv_opensource.utils import ievv_colorize

            print(ievv_colorize('Test', color=ievv_colorize.COLOR_BLUE))

        Print bold red text::

            print(ievv_colorize('Test', color=ievv_colorize.COLOR_RED, bold=True))

    Args:
        text: The text (string) to colorize.
        color: The color to use.
            Should be one of:

            - :obj:`.COLOR_RED`
            - :obj:`.COLOR_BLUE`
            - :obj:`.COLOR_YELLOW`
            - :obj:`.COLOR_GREY`
            - :obj:`.COLOR_GREEN`
            - ``None`` (no color)
        bold: Set this to ``True`` to use bold font.
    """
    if getattr(settings, 'IEVV_COLORIZE_USE_COLORS', True) and color:
        attrs = []
        if bold:
            attrs.append('bold')
        return colored(text, color=color, attrs=attrs)
    else:
        return text



import os
from ievv_opensource.utils.desktopnotifications.base import AbstractNotification

try:
    from shlex import quote
except ImportError:
    from pipes import quote


class Notification(AbstractNotification):
    def show_message(self, title, message):
        command = 'display notification "{message}" with title "{title}"'.format(
            title=title.replace('"', "'"),
            message=message.replace('"', "'"))
        os.system('osascript -e {}'.format(quote(command)))



from django.contrib.postgres.fields import JSONField
from django.db import models


class IevvLoggingEventBase(models.Model):
    slug = models.CharField(max_length=255, unique=True, db_index=True)
    last_started = models.DateTimeField()
    last_finished = models.DateTimeField(null=True, blank=True)
    time_spent = models.CharField(max_length=255, null=False, blank=True, default='')
    time_spent_in_seconds = models.IntegerField(null=True, blank=True)
    success_run = models.NullBooleanField(default=None)

    def __str__(self):
        return self.slug

    class Meta:
        ordering = ['-id']


class IevvLoggingEventItem(models.Model):
    logging_base = models.ForeignKey(IevvLoggingEventBase, on_delete=models.CASCADE)
    time_spent = models.CharField(max_length=255, null=False, blank=True, default='')
    time_spent_in_seconds = models.IntegerField(null=True, blank=True)
    success_run = models.NullBooleanField(default=None)
    data = JSONField(null=True, blank=True)
    start_datetime = models.DateTimeField()
    end_datetime = models.DateTimeField(null=True, blank=True)
    created_datetime = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f'{self.logging_base}'

    class Meta:
        ordering = ['-id']



import multiprocessing
import threading
from collections import OrderedDict

import psutil
from watchdog.events import RegexMatchingEventHandler
from watchdog.observers import Observer

from ievv_opensource.utils.logmixin import LogMixin


class WatchdogWatchConfig(object):
    """
    Used by plugins to configure watching.

    See :meth:`ievv_opensource.utils.ievvbuildstatic.pluginbase.Plugin.watch`.
    """
    def run(self):
        self.plugin.run()

    def __init__(self, watchfolders, watchregexes, plugin):
        """
        Args:
            watchfolders: List of folders to watch.
            watchregexes: List of regexes to watch.
            plugin: A :class:`ievv_opensource.utils.ievvbuildstatic.pluginbase.Plugin` object.
        """
        self.folders = watchfolders
        self.regexes = watchregexes
        self.plugin = plugin


class ProcessWrapper(object):
    def __init__(self, process):
        self.process = process
        self.process.daemon = True

    def start(self):
        self.process.start()

    def stop(self):
        try:
            process = psutil.Process(self.process.pid)
        except psutil.NoSuchProcess:
            pass
        else:
            for childprocess in process.children():
                childprocess.terminate()
            try:
                process.terminate()
            except psutil.NoSuchProcess:
                pass

    def join(self):
        self.process.join()


class ProcessWatchConfig(object):
    """

    """
    def __init__(self, plugin):
        """
        Args:
            plugin: A :class:`ievv_opensource.utils.ievvbuildstatic.pluginbase.Plugin` object.
        """
        self.plugin = plugin

    def create_process(self):
        process = multiprocessing.Process(target=self.plugin.run_watcher_process)
        return ProcessWrapper(process=process)


class WatchConfigPool(LogMixin):
    def __init__(self):
        self._watchconfigs = []

    def extend(self, watchconfigs):
        self._watchconfigs.extend(watchconfigs)

    def get_logger_name(self):
        return 'watcherpool'

    def __watch_folder(self, folderpath, plugins, regexes):
        event_handler = EventHandler(
            plugins=plugins,
            regexes=regexes
        )
        observer = Observer()
        observer.schedule(event_handler, folderpath, recursive=True)
        self.get_logger().info(
            'Starting watcher for folder {!r} with regexes {!r}. '
            'Runnables notified of changes: {}.'.format(
                folderpath, regexes, ', '.join(str(plugin) for plugin in plugins)))
        observer.start()
        return observer

    def __build_foldermap(self):
        # Note: We use OrderedDict here, and a list (instead of set) for
        #       plugins below to maintain the order of apps when building.
        #       This should not matter, but it is nice to have the apps
        #       build in the same order as they are listed in settings.
        watchdog_foldermap = OrderedDict()
        process_runnables = []
        for watchconfig in self._watchconfigs:
            if isinstance(watchconfig, WatchdogWatchConfig):
                for folderpath in watchconfig.folders:
                    if folderpath not in watchdog_foldermap:
                        watchdog_foldermap[folderpath] = {
                            'regexes': set(),
                            'plugins': []
                        }
                    watchdog_foldermap[folderpath]['regexes'].update(watchconfig.regexes)
                    if watchconfig.plugin not in watchdog_foldermap[folderpath]['plugins']:
                        watchdog_foldermap[folderpath]['plugins'].append(watchconfig.plugin)
            elif isinstance(watchconfig, ProcessWatchConfig):
                process_runnables.append(watchconfig)
            else:
                raise ValueError('Unknown watch config type: {}'.format(type(watchconfig)))
        return watchdog_foldermap, process_runnables

    def watch(self):
        watchdog_foldermap, process_runnables = self.__build_foldermap()
        observers = []
        for folderpath, folderkwargs in watchdog_foldermap.items():
            observer = self.__watch_folder(folderpath=folderpath, **folderkwargs)
            observers.append(observer)
        for process_runnable in process_runnables:
            process = process_runnable.create_process()
            process.start()
            observers.append(process)
        return observers


class EventHandler(RegexMatchingEventHandler):
    """
    Event handler for watchdog --- this is used by each watcher
    thread to react to changes in the filesystem.

    This is instantiated by :meth:`ievv_opensource.utils.ievvbuildstatic.pluginbase.Plugin.watch`
    to watch for changes in files matching
    :meth:`ievv_opensource.utils.ievvbuildstatic.pluginbase.Plugin.get_watch_regexes`
    in the folders specified in
    :meth:`ievv_opensource.utils.ievvbuildstatic.pluginbase.Plugin.get_watch_folders`.
    """
    def __init__(self, *args, **kwargs):
        self.plugins = kwargs.pop('plugins')
        self.runtimer = None
        self.is_running = False
        super(EventHandler, self).__init__(*args, **kwargs)

    def handle_plugin_run(self):
        if self.is_running:
            return
        self.is_running = True
        for plugin in self.plugins:
            plugin.runwrapper()

        self.is_running = False

    def on_any_event(self, event):
        if self.runtimer:
            self.runtimer.cancel()
        self.runtimer = threading.Timer(0.5, self.handle_plugin_run)
        self.runtimer.start()



import os
import shutil

from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'Remove the sorl-thumbnail cache media directory if there is one.'

    def handle(self, *args, **options):
        thumbnail_prefix = getattr(settings, 'THUMBNAIL_PREFIX', None)
        if thumbnail_prefix:
            cache_directory = os.path.join(settings.MEDIA_ROOT, thumbnail_prefix)
            if os.path.exists(cache_directory):
                shutil.rmtree(cache_directory)
                self.stdout.write('Removed sorl-cache directory: {}'.format(
                    cache_directory))
            else:
                self.stdout.write('Sorl-cache directory does not exist: {}'.format(
                    cache_directory))



from __future__ import absolute_import

import django_rq

from ievv_opensource.ievv_batchframework.models import BatchOperation
from ievv_opensource.ievv_batchframework import batchregistry

import logging


class BatchActionGroupTask(object):
    abstract = True

    def run_actiongroup(self, actiongroup_name, batchoperation_id, **kwargs):
        try:
            batchoperation = BatchOperation.objects\
                .get(id=batchoperation_id, status=BatchOperation.STATUS_UNPROCESSED)
        except BatchOperation.DoesNotExist:
            logging.warning('BatchOperation with id={} does not exist, or is already running.')
            return
        else:
            batchoperation.mark_as_running()
            registry = batchregistry.Registry.get_instance()
            full_kwargs = {
                'started_by': batchoperation.started_by,
                'context_object': batchoperation.context_object,
            }
            full_kwargs.update(kwargs)
            actiongroupresult = registry.get_actiongroup(actiongroup_name)\
                .run_blocking(**full_kwargs)
            batchoperation.finish(failed=actiongroupresult.failed,
                                  output_data=actiongroupresult.to_dict())


@django_rq.job('default')
def default(**kwargs):
    BatchActionGroupTask().run_actiongroup(**kwargs)


@django_rq.job('highpriority')
def highpriority(**kwargs):
    BatchActionGroupTask().run_actiongroup(**kwargs)



import json
import logging
import traceback
from collections import OrderedDict

from django.conf import settings

from ievv_opensource.utils.singleton import Singleton


class ActionResult(object):
    def __init__(self, actionclass):
        self.actionclass = actionclass
        self.failed = False
        self.success_output_data = None
        self.exception_traceback = None
        self.exception = None

    def set_exception(self, exception, exception_traceback):
        self.failed = True
        self.exception = exception
        self.exception_traceback = exception_traceback

    def set_success_output_data(self, success_output_data):
        self.success_output_data = success_output_data

    def get_error_data(self):
        if isinstance(self.exception, ActionError):
            return self.exception.error_data_dict
        else:
            return str(self.exception)

    def get_exception_traceback(self):
        return self.exception_traceback

    def get_errordata(self):
        if self.exception:
            return {
                'data': self.get_error_data(),
                'traceback': self.get_exception_traceback()
            }
        else:
            return None

    def to_dict(self):
        return {
            'action_name': self.actionclass.get_name(),
            'failed': self.failed,
            'errordata': self.get_errordata(),
            'success_output_data': self.success_output_data
        }


class ActionGroupResult(object):
    def __init__(self, actiongroup):
        self.actiongroup = actiongroup
        self._actionresults = OrderedDict()

    def __getitem__(self, actiongroup_name):
        return self._actionresults[actiongroup_name]

    def __iter__(self):
        return self._actionresults.values()

    @property
    def failed(self):
        for actionresult in self._actionresults.values():
            if actionresult.failed:
                return True
        return False

    def add_actionresult(self, actionresult):
        self._actionresults[actionresult.actionclass.get_name()] = actionresult

    def to_dict(self):
        resultlist = []
        for actionresult in self._actionresults.values():
            resultlist.append(actionresult.to_dict())
        return {
            'actiongroup_name': self.actiongroup.name,
            'results': resultlist
        }

    def __str__(self):
        return json.dumps(self.to_dict(), indent=4)


class ActionGroupSynchronousExecutionError(Exception):
    def __init__(self, actiongroupresult):
        self.actiongroupresult = actiongroupresult

    def __str__(self):
        return json.dumps(self.actiongroupresult.to_dict(), indent=4)


class ActionError(Exception):
    def __init__(self, error_data_dict):
        self.error_data_dict = error_data_dict

    def __str__(self):
        return json.dumps(self.error_data_dict, indent=4)

    def __repr__(self):
        return str(self)


def action_factory(baseclass, name):
    """
    Factory for creating :class:`.Action` classes. This is simply
    a thin wrapper around ``type`` to dynamically create a subclass
    of the given ``baseclass`` with a different name.

    There are two use cases for using this:

    - Give re-usable action classes a better name.
    - Use the same :class:`.Action` subclass multiple times in the same
      :class:`.ActionGroup`.

    Both of these cases can also be solved with subclasses, and that
    is normally a better solution.

    Args:
        baseclass: The class to create a subclass of.
        name: The name of the subclass.
    """
    return type(name, (baseclass,), {})


class Action(object):
    """
    An action is the subclass for code that can be executed as
    part of an :class:`.ActionGroup`.

    You create a subclass of this class, and override :meth:`.exectute`
    to implement an action, and you add your subclass to an
    :class:`.ActionGroup` to use your action class.
    """
    @classmethod
    def run(cls, **kwargs):
        """
        Run the action - used internally.

        Args:
            kwargs: Kwargs for the action.
            executed_by_celery: Must be ``True`` if the action is executed by a celery task.
                This is required to configure the correct logger.
        """
        action = cls(**kwargs)
        action.execute()

    @classmethod
    def get_name(cls):
        """
        Get the name of this action.
        """
        return '{}.{}'.format(
            cls.__module__,
            cls.__name__)

    def __init__(self, **kwargs):
        self.kwargs = kwargs
        # self.logger = kwargs['logger']

    @property
    def logger(self):
        """
        Get the logger for this action.
        """
        logname = self.__class__.get_name()
        return logging.getLogger(logname)

    @property
    def executed_by_celery(self):
        return self.kwargs.get('executed_by_celery', False)

    @property
    def context_object(self):
        return self.kwargs['context_object']

    @property
    def started_by(self):
        return self.kwargs['started_by']

    def execute(self):
        """
        Execute the action. Must be overridden in subclasses.
        """
        raise NotImplementedError()


class ActionGroupExecutionInfo(object):
    """
    Return value from :meth:`.ActionGroup.run`, with information about
    how the ActionGroup was executed.

    .. attribute:: actiongroup
        The :class:`.ActionGroup` object that was executed.

    .. attribute:: mode
        The mode the ActionGroup was executed with.

    .. attribute:: route_to_alias
        The route_to_alias that was used to route/prioritize the execution of
        the ActionGroup.
    """
    def __init__(self, actiongroup, mode, route_to_alias,
                 actiongroupresult=None, batchoperation=None):
        self.actiongroup = actiongroup
        self.mode = mode
        self.route_to_alias = route_to_alias
        self._actiongroupresult = actiongroupresult
        self._batchoperation = batchoperation

    @property
    def is_asynchronous(self):
        """
        Property that returns ``True`` if the ActionGroup was executed
        asynchronously.
        """
        return self.mode == ActionGroup.MODE_ASYNCHRONOUS

    @property
    def actiongroupresult(self):
        """
        Property for getting the :class:`.ActionGroupResult` if the
        ActionGroup was executed in synchronous mode.

        Raises:
            AttributeError: If ``mode`` is :obj:`.ActionGroup.MODE_ASYNCHRONOUS`.
        """
        if self.is_asynchronous:
            raise AttributeError(
                'Can not use ActionGroupExecutionInfo.actiongroupresult when execution mode '
                'is asynchronous. In asynchronous mode, this is stored in a BatchOperation object '
                'when the asynchronous operation is finished. You can get BatchOperation object '
                'using the batchoperation property.'
            )
        else:
            return self._actiongroupresult

    @property
    def batchoperation(self):
        """
        Property for getting the :class:`ievv_opensource.ievv_batchframework.models.BatchOperation`
        object that was created if the ActionGroup was executed in asynchronous mode.

        Raises:
            AttributeError: If ``mode`` is :obj:`.ActionGroup.MODE_ASYNCHRONOUS`.
        """
        if self.is_asynchronous:
            return self._batchoperation
        else:
            raise AttributeError(
                'Can not use ActionGroupExecutionInfo.batchoperation when execution mode '
                'is synchronous. In synchronous mode, you can use the actiongroupresult '
                'property to access the results.'
            )

    def __str__(self):
        common_str = '{}.{}(mode={!r}, route_to_alias={!r})'.format(
            self.actiongroup.__class__.__module__,
            self.actiongroup.__class__.__name__,
            self.mode, self.route_to_alias)
        if self.is_asynchronous:
            return '{} - BatchOperation({})'.format(common_str, self.batchoperation)
        else:
            return '{} - ActionGroupResult({})'.format(common_str, self.actiongroupresult)


class ActionGroup(object):
    """
    An ActionGroup is a list of :class:`actions <.Action>` that can be
    executed both synchronously and asynchronously.
    """

    #: Constant for asynchronous (background/Celery) mode of execution.
    MODE_ASYNCHRONOUS = 'asynchronous'

    #: Constant for synchronous (blocking) mode of execution.
    MODE_SYNCHRONOUS = 'synchronous'

    def __init__(self, name, actions=None, mode=None, route_to_alias=None):
        """
        Args:
            name: The name of the ActionGroup.
            actions: A list of actions.
            mode: The default mode of the ActionGroup.
                Defaults to :obj:`~.ActionGroup.MODE_ASYNCHRONOUS`.
                You will often want to determine this from the input
                (I.E.: Use asynchronous if sending more than 500 newsletters),
                and this can be done by extending this class and overriding
                :meth:`.get_mode`.
            route_to_alias: Defines where to route this ActionGroup when
                is is executed in asynchronous mode. Defaults to
                :obj:`.Registry.ROUTE_TO_ALIAS_DEFAULT`.
                You can determine the route dynamically each time the ActionGroup
                is executed by overriding :meth:`.get_route_to_alias`.
        """
        self.name = name
        self.mode = mode or self.MODE_ASYNCHRONOUS
        # self.route_to_alias = route_to_alias or Registry.ROUTE_TO_ALIAS_DEFAULT
        self.route_to_alias = route_to_alias or Registry.ROUTE_TO_TASK_ALIAS_DEFAULT
        self.actions = OrderedDict()
        self.registry = None  # Set when the ActionGroup is added to the Registry
        if actions:
            self.add_actions(actions=actions)

    def add_action(self, action):
        """
        Add a action.

        Args:
            action: A subclass of :class:`.Action` (not an object, but a class).
        """
        name = action.get_name()
        if name in self.actions:
            raise ValueError(
                'Duplicate action class in one ActionGroup: {name}. '
                'You can not register more than one action of the same class in the same '
                'ActionGroup. Create multiple subclasses of {name}, or use batchregistry.action_factory: '
                'actions=[batchregistry.action_factory({classname}, name="FirstAction"), '
                'batchregistry.action_factory({classname}, name="SecondAction")].'.format(
                    name=name, classname=action.__name__))
        self.actions[name] = action

    def add_actions(self, actions):
        """
        Add actions.

        Args:
            actions: A list of :class:`.Action` subclasses (classes not actions).
        """
        for action in actions:
            self.add_action(action=action)

    def get_mode(self, **kwargs):
        """
        Get the mode to run the ActionGroup in.
        Must return one of :obj:`~.ActionGroup.MODE_ASYNCHRONOUS` or :obj:`~.ActionGroup.MODE_SYNCHRONOUS`.

        The use-case for overriding this method is optimization. Lets say
        you have to re-index your blogposts in a search engine each time they
        are updated. If you update just a few blogpost, you may want to do
        that in synchronous mode, but if you update 500 blogposts, you will
        probably want to re-index in asynchronous mode (I.E. in Celery).

        Args:
            kwargs: The ``kwargs`` the user provided to :meth:`.run`.
        """
        return self.mode

    def get_route_to_alias(self, **kwargs):
        """
        Define where to route this ActionGroup when is is executed in asynchronous mode.

        This is the method you want to override to handle priority of your
        asynchronously executed ActionGroups.

        Lets say you have a huge blog, with lots of
        traffic. After updating a blogpost, you need to do some heavy postprocessing
        (image optimization, video transcoding, etc.). If you update a newly posted blogpost
        this postprocessing should be placed in a high-priority queue, and if you update an old
        blogpost, this postprocessing should be placed in a low-priority queue. To achieve this,
        you simply need to create a subclass of ActionGroup, and override this method
        to return :obj:`.Registry.ROUTE_TO_ALIAS_HIGHPRIORITY` for recently created blogpost, and
        :obj:`.Registry.ROUTE_TO_ALIAS_DEFAULT` for old blogposts.

        Args:
            kwargs: The ``kwargs`` the user provided to :meth:`.run_asynchronous`.

        Returns:
            str: One of the route-to aliases added to the :class:`.Registry`
            using :meth:`.Registry.add_route_to_alias`. This will always include
            :obj:`.Registry.ROUTE_TO_ALIAS_DEFAULT` and :obj:`.Registry.ROUTE_TO_ALIAS_HIGHPRIORITY`.
        """
        return self.route_to_alias

    def run_blocking(self, **kwargs):
        # This method is for internal use only.
        actiongroupresult = ActionGroupResult(actiongroup=self)
        for actionclass in self.actions.values():
            actionresult = ActionResult(actionclass=actionclass)
            try:
                success_output_data = actionclass.run(**kwargs)
            except Exception as exception:
                exception_traceback = traceback.format_exc()
                actionresult.set_exception(
                    exception=exception,
                    exception_traceback=exception_traceback)
            else:
                actionresult.set_success_output_data(
                    success_output_data=success_output_data)
            actiongroupresult.add_actionresult(actionresult=actionresult)
        return actiongroupresult

    def run_synchronous(self, **kwargs):
        """
        Run the ActionGroup in blocking/synchronous mode.

        Args:
            kwargs: Kwargs for :class:`.Action`.
        """
        actiongroupresult = self.run_blocking(**kwargs)
        if actiongroupresult.failed:
            raise ActionGroupSynchronousExecutionError(actiongroupresult=actiongroupresult)
        else:
            return ActionGroupExecutionInfo(
                actiongroup=self,
                mode=self.MODE_SYNCHRONOUS,
                route_to_alias=None,
                actiongroupresult=actiongroupresult
            )

    def get_batchoperation_options(self, **kwargs):
        """
        You can override this if you create a re-usable ActionGroup subclass that
        sets options for the :class:`ievv_opensource.ievv_batchframework.models.BatchOperation`
        based on ``kwargs``.

        Called by :meth:`.run_asynchronous` to get the kwargs for
        :meth:`ievv_opensource.ievv_batchframework.models.BatchOperationManager.create_asynchronous`.

        If you override this, you should normally call ``super()``, and update the kwargs
        returned by super.

        Args:
            kwargs: The ``kwargs`` the user provided to :meth:`.run_asynchronous`.

        Returns:
            dict: Kwargs for
            :meth:`ievv_opensource.ievv_batchframework.models.BatchOperationManager.create_asynchronous`
        """
        options = {
            'operationtype': self.name,
            'context_object': kwargs.get('context_object', None),
            'started_by': kwargs.get('started_by', None),
        }
        return options

    def create_batchoperation(self, **kwargs):
        """
        Used by :meth:`.run_asynchronous` to create the
        :class:`ievv_opensource.ievv_batchframework.models.BatchOperation` object.

        You normally do not need to override this - override :meth:`.get_batchoperation_options`
        instead.

        .. warning:: Overriding this may lead to breaking code if the inner workings of this framework
            is changed/optimized in the future.

        Args:
            kwargs: See :meth:`.run_asynchronous`.

        Returns:
            BatchOperation: The created :class:`ievv_opensource.ievv_batchframework.models.BatchOperation`.
        """
        from ievv_opensource.ievv_batchframework.models import BatchOperation
        batchoperation_options = self.get_batchoperation_options(**kwargs)
        batchoperation = BatchOperation.objects.create_asynchronous(**batchoperation_options)
        return batchoperation

    def run_asynchronous(self, **kwargs):
        """
        Args:
            kwargs: Kwargs for :class:`.Action`.
        """
        batchoperation = self.create_batchoperation(**kwargs)
        full_kwargs = {
            'actiongroup_name': self.name,
            'batchoperation_id': batchoperation.id
        }
        full_kwargs.update(kwargs)

        # Remove the kwargs that is stored in the batchoperation
        full_kwargs.pop('context_object', None)
        full_kwargs.pop('started_by', None)

        route_to_alias = self.get_route_to_alias(**kwargs)
        self.registry.queue_to_task_map[route_to_alias].delay(
            **full_kwargs
        )
        return ActionGroupExecutionInfo(
            actiongroup=self,
            mode=self.MODE_ASYNCHRONOUS,
            route_to_alias=route_to_alias,
            batchoperation=batchoperation)

    def run(self, **kwargs):
        """
        Runs one of :meth:`.run_asynchronous` and :meth:`.run_synchronous`. The method to
        run is determined by the return-value of :meth:`.get_mode`:

        - If :meth:`.get_mode` returns :obj:`~.ActionGroup.MODE_ASYNCHRONOUS`, :meth:`.run_asynchronous`
          is called.
        - If :meth:`.get_mode` returns :obj:`~.ActionGroup.MODE_SYNCHRONOUS`, :meth:`.run_synchronous`
          is called.

        Args:
            context_object: context_object for :class:`ievv_opensource.ievv_batchframework.models.BatchOperation`.
            started_by: started_by for :class:`ievv_opensource.ievv_batchframework.models.BatchOperation`.
            **kwargs: Kwargs for :class:`.Action`. Forwarded to :meth:`.run_asynchronous` and
                :meth:`.run_synchronous`.
        """
        if getattr(settings, 'IEVV_BATCHFRAMEWORK_ALWAYS_SYNCRONOUS', False):
            mode = self.MODE_SYNCHRONOUS
        else:
            mode = self.get_mode(**kwargs)
        if mode == self.MODE_ASYNCHRONOUS:
            return self.run_asynchronous(**kwargs)
        else:
            return self.run_synchronous(**kwargs)


class Registry(Singleton):
    """
    The registry of :class:`.ActionGroup` objects.
    """
    ROUTE_TO_TASK_ALIAS_DEFAULT = 'default'
    ROUTE_TO_TASK_ALIAS_HIGHPRIORITY = 'highpriority'

    def __init__(self):
        self.actiongroups = OrderedDict()
        self.queue_to_task_map = {}
        self.__add_default_route_to_aliases()
        super(Registry, self).__init__()

    def __add_default_route_to_aliases(self):
        from ievv_opensource.ievv_batchframework.rq_tasks import default, highpriority
        self.add_route_to_alias(
            route_to_alias=self.ROUTE_TO_TASK_ALIAS_DEFAULT,
            task_callable=default,
        )
        self.add_route_to_alias(
            route_to_alias=self.ROUTE_TO_TASK_ALIAS_HIGHPRIORITY,
            task_callable=highpriority,
        )

    def add_route_to_alias(self, route_to_alias, task_callable):
        """
        Add a route-to alias.

        Args:
            route_to_alias (str): The alias.
            task_callable (func): The callable rq task.
        """
        self.queue_to_task_map[route_to_alias] = task_callable


    def add_actiongroup(self, actiongroup):
        """
        Add an :class:`.ActionGroup` to the registry.

        Args:
            actiongroup: The :class:`.ActionGroup` object to add.
        """
        actiongroup.registry = self
        self.actiongroups[actiongroup.name] = actiongroup

    def remove_actiongroup(self, actiongroup_name):
        """
        Remove an :class:`ActionGroup` from the registry.

        Args:
            actiongroup_name: The name of the actiongroup.

        Raises:
            KeyError: If no :class:`.ActionGroup` with the provided
                ``actiongroup_name`` exists in the registry.
        """
        del self.actiongroups[actiongroup_name]

    def get_actiongroup(self, actiongroup_name):
        """
        Get an :class:`.ActionGroup` object from the registry.

        Args:
            actiongroup_name: The name of the actiongroup.

        Returns:
            ActionGroup: An :class:`.ActionGroup` object.

        Raises:
            KeyError: If no :class:`.ActionGroup` with the provided
                ``actiongroup_name`` exists in the registry.
        """
        return self.actiongroups[actiongroup_name]

    def run(self, actiongroup_name, **kwargs):
        """
        Shortcut for::

            Registry.get_instance().get_actiongroup(actiongroup_name)\
                .run(**kwargs)

        .. seealso:: :meth:`.get_actiongroup` and :meth:`.ActionGroup.run`.
        """
        return self.get_actiongroup(actiongroup_name=actiongroup_name).run(**kwargs)



from django.http import HttpResponse


def unnamed_untranslated_exampleview(request, *args, **kwargs):
    return HttpResponse('unnamed untranslated')


def unnamed_translated_exampleview(request, *args, **kwargs):
    return HttpResponse('unnamed translated')


def named_untranslated_exampleview(request, *args, **kwargs):
    return HttpResponse('named untranslated')


def named_translated_exampleview(request, *args, **kwargs):
    return HttpResponse('named translated')



from django.contrib import admin

from ievv_opensource.ievv_batchframework.models import BatchOperation


@admin.register(BatchOperation)
class BatchOperationAdmin(admin.ModelAdmin):
    list_per_page = 20

    list_display = [
        'id',
        'status',
        'operationtype',
        'started_by',
        'created_datetime',
        'started_running_datetime',
        'finished_datetime',
        'context_object',
    ]

    readonly_fields = [
        'input_data_json',
        'output_data_json',
        'result',
        'status',
    ]

    list_filter = [
        'status',
        'created_datetime',
        'started_running_datetime',
        'finished_datetime',
    ]



from collections import OrderedDict


class Choice(object):
    """
    A choice in a :class:`.ChoicesWithMeta`.

    This basic choice class supports value, label and description,
    but you should subclass this (and possibly also :class:`.ChoicesWithMeta`)
    if you need more metadata.

    .. attribute:: value

        The value which is typically stored in the database, or
        sent as the actual POST data value in forms.

    .. attribute:: label

        A short user-friendly label for the choice.

    .. attribute:: description

        A user-friendly longer description of the choice.
    """
    def __init__(self, value, label=None, description='',
                 attributename=None):
        """
        Args:
            value: The value for the choice.
                The value which is typically stored in the database, or
                sent as the actual POST data value in forms.
            label: A user-friendly short label for the choice.
                This is normally marked for translation.
                Defaults to ``value`` if ``bool(label) == False``.
            description: A user-friendly longer description of the choice.
                This is normally marked for translation.
                Not required.
        """
        self.value = value
        self.label = label or value
        self.description = description
        if attributename is None:
            self.attributename = self._value_to_attributename(value)
        else:
            self.attributename = attributename

    @classmethod
    def get_classpath(cls):
        return f'{cls.__module__}.{cls.__name__}'

    @property
    def classpath(self):
        return self.__class__.get_classpath()

    def _value_to_attributename(self, value):
        valuestring = str(value)
        attributename = valuestring.upper().replace('-', '_').replace(' ', '_')
        if len(attributename) > 0 and attributename[0].isdigit():
            attributename = '_{}'.format(attributename)
        return attributename

    def get_short_label(self):
        """
        Get a short label for the choice.

        Defaults to returning :attr:`~.Choice.label`, but you
        can override this in subclasses.
        """
        return self.label

    def get_long_label(self):
        """
        Get a long label for the choice.

        Generating by joining :attr:`~.Choice.label` and :attr:`~.Choice.description`
        with ``" - "``.
        """
        if self.description:
            return '{} - {}'.format(self.label, self.description)
        else:
            return self.label

    def __str__(self):
        return self.value

    @property
    def translated_label(self):
        from django.utils.translation import ugettext
        return ugettext(self.label)

    @property
    def translated_description(self):
        from django.utils.translation import ugettext
        return ugettext(self.description)

    def as_serializable_data(self):
        return {
            'value': self.value,
            'label': self.translated_label,
            'description': self.translated_description
        }


class ChoicesWithMeta(object):
    """
    An object oriented structure for model/form field choices.

    Unlike the simple ``(value, label)`` tuple used in
    Django, this miniframework supports more metadata because
    the choices are defined through :class:`.Choice` (which you can subclass
    and extend).

    Compatible with the ``choices``-attribute used in Django
    (I.E.: django.forms.ChoiceField, django.forms.CharField, ...)
    through the :meth:`~.ChoicesWithMeta.iter_as_django_choices_short`
    and :meth:`~.ChoicesWithMeta.iter_as_django_choices_long` methods.

    Examples:

        Usage in Django model::

            class User(models.Model):
                username = models.CharField(max_length=255, unique=True)

                USERTYPE_CHOICES = choices_with_meta.ChoicesWithMeta(
                    choices_with_meta.Choice(value='normal', label='Normal'),
                    choices_with_meta.Choice(value='editor', label='Editor'),
                    choices_with_meta.Choice(value='admin', label='Admin')
                )

                usertype = models.CharField(
                    max_length=255,
                    choices=USERTYPE_CHOICES.iter_as_django_choices_short,
                    default=USERTYPE_CHOICES.NORMAL
                )

        Lets say you want to provivide a bit more information about
        each choice. Then you can use the ``description`` parameter
        for :class:`.Choice`::

            choices_with_meta.Choice(
                value='admin',
                label='Admin',
                description='An administrator user with access to everything.')

        You will most likely also want to update the ``choices``-argument for
        your choice field to use :meth:`~.ChoicesWithMeta.iter_as_django_choices_long`
        (to include description in the default label), or you may just want
        to do that in certain views. You can also extend the :class:`Choice class<.Choice>`
        and add your own logic in :meth:`.get_long_label` and :meth:`.get_short_label`.

        ChoicesWithMeta makes it easier to provide good exception messages::

            if "somechoice" not in User.USERTYPE_CHOICES:
                raise ValidationError({
                    'usertype': 'Must be one of: {}'.format(
                        User.USERTYPE_CHOICES.get_values_as_commaseparated_string()
                    )
                })

        Getting choices by value is easy::

            User.USERTYPE_CHOICES['admin'].label

        You can access values as an attribute of the ChoicesWithMeta object
        as the value uppercased with ``-`` and space replaced with ``_``::

            User.USERTYPE_CHOICES.ADMIN.label

        Getting choices by index is also easy::

            User.USERTYPE_CHOICES.get_choice_at_index(1).label

        Getting the first choice (typically the default choice) is easy::

            User.USERTYPE_CHOICES.get_first_choice()

        You can iterate over the choices or values::

            for choice in User.USERTYPE_CHOICES.iterchoices():
                pass

            for value in User.USERTYPE_CHOICES.itervalues():
                pass

        You can get the values as a list or as a comma separated string::

            User.USERTYPE_CHOICES.get_values_as_list()
            User.USERTYPE_CHOICES.get_values_as_commaseparated_string()
    """
    def __init__(self, *choices):
        self.choices = OrderedDict()
        for choice in self.get_default_choices():
            self.add(choice)
        for choice in choices:
            self.add(choice)

    def get_by_value(self, value, fallback=None):
        """
        Get the :class:`.Choice` with the provided ``value``.

        Args:
            value: The value to lookup.
            fallback: Fallback value if ``value`` is not registered as a choice value.

        Returns:
            .Choice: The Choice matching the value if it exists, otherwise return ``fallback``.
        """
        return self.choices.get(value, fallback)

    def __getitem__(self, value):
        """
        Get the :class:`.Choice` with the provided ``value``.

        Raises:
            KeyError: If no :class:`.Choice` with the provided
                ``value`` is in the ChoicesWithMeta.
        """
        return self.choices[value]

    def __contains__(self, value):
        """
        Check if ``value`` is one of the choices.

        Returns:
            bool: True a :class:`.Choice` with the provided ``value``
                is in the ChoicesWithMeta.
        """
        return value in self.choices

    def __len__(self):
        """
        Get the number of choices.
        """
        return len(self.choices)

    def get_default_choices(self):
        """
        Lets say you have a field where a set of default choices
        make sense. You then create a subclass of ChoicesWithMeta
        and override this method to return these default choices.

        Developers can still override this method for special
        cases, but the default will be that these default choices
        are included.

        This is most useful when choices are a Django setting
        that users of your app can override.

        Returns:
            iterable: An iterable of :class:`.Choice` objects.
        """
        return []

    def get_choice_at_index(self, index):
        """
        Args:
            index: The numeric index of the choice.

        Raises:
            IndexError: If the index does not correspond to a choice.

        Returns:
            .Choice: The :class:`.Choice` at the provided index.
        """
        keys = list(self.choices.keys())
        value = keys[index]
        return self.choices[value]

    def get_first_choice(self):
        """
        Uses :meth:`.get_choice_at_index` to get the first (index=0) choice.
        If there is no first choice, ``None`` is returned.
        """
        try:
            return self.get_choice_at_index(0)
        except IndexError:
            return None

    def add(self, choice):
        """
        Add a :class:`.Choice`.

        Args:
            choice: A :class:`.Choice` object.
        """
        if choice.value in self.choices:
            raise KeyError('A choice with value "{}" alredy exists.'.format(choice.value))
        self.choices[choice.value] = choice
        setattr(self, choice.attributename, choice)

    def remove(self, value):
        """
        Remove a choice by value.

        Args:
            value: The value to remove from the ChoicesWithMeta.
        """
        if value in self.choices:
            attributename = self.choices[value].attributename
            delattr(self, attributename)
            del self.choices[value]
        else:
            raise KeyError('{value} is not a valid choice value.'.format(value=value))

    def itervalues(self):
        """
        Iterate over the choices yielding only the values in the added order.
        """
        return self.choices.keys()

    def iterchoices(self, *extra_choices):
        """
        Iterate over the choices yielding :class:`.Choice` objects in the added order.

        Args:
            *extra_choices: Extra choices. Zero or more extra choices (:class:`.Choice` objects) to
                yield at the end. Can be used in cases where you allow invalid choices if they are
                already set (so you want to include the current invalid choice along with the
                correct choices).
        """
        for choice in self.choices.values():
            yield choice
        for choice in extra_choices:
            yield choice

    def iter_as_django_choices_short(self, *extra_choices):
        """
        Iterate over the choices as a Django choices list,
        where each item is a ``(value, label)``-tuple.

        Uses :meth:`.Choice.get_short_label` to create the ``label``.

        Args:
            *extra_choices: See :meth:`.iterchoices`.
        """
        for choice in self.iterchoices(*extra_choices):
            yield choice.value, choice.get_short_label()

    def iter_as_django_choices_long(self, *extra_choices):
        """
        Iterate over the choices as a Django choices list,
        where each item is a ``(value, label)``-tuple.

        Uses :meth:`.Choice.get_long_label` to create the ``label``.

        Args:
            *extra_choices: See :meth:`.iterchoices`.
        """
        for choice in self.iterchoices(*extra_choices):
            yield choice.value, choice.get_long_label()

    def get_values_as_list(self):
        """
        Get the values of all choices in the added order as a list.
        """
        return list(self.itervalues())

    def get_values_as_commaseparated_string(self):
        """
        Get the values as a comma-separated string.

        Perfect for showing available choices in error messages.

        Returns:
            String with all the values separated by comma.
        """
        return ', '.join(self.itervalues())

    def __str__(self):
        return 'ChoicesWithMeta({})'.format(self.get_values_as_commaseparated_string())



import arrow
from django.conf import settings
from django.template import defaultfilters


def format_datetime_in_timezone(datetime_object, datetime_format='DATETIME_FORMAT', timezone=settings.TIME_ZONE):
    """
    Format a datetime object in a timezone.

    Args:
        datetime_object (datetime.datetime): Datetime object to format.
        datetime_format (str): A Django datetime formatting string name, such as ``"DATETIME_FORMAT"``,
            ``"SHORT_DATETIME_FORMAT``", ``"DATE_FORMAT"``, ...
        timezone (str): Defaults to ``settings.TIME_ZONE``. The datetime is converted to
            this timezone. So if you use UTC in the database, and want to present another
            timezone, this will convert it correctly.

    Returns:
        str: The formatted datetime.
    """
    return defaultfilters.date(arrow.get(datetime_object).to(timezone).datetime, datetime_format)





class AbstractNotification(object):
    def show_message(self, title, message):
        """
        Show message in the notification system of the OS.

        Parameters:
            title: The title of the notification.
            message: The notification message.
        """
        raise NotImplementedError()



import re

from django.conf import settings
from django.utils.module_loading import import_string


class BasePhoneNumberHandler:
    STRIP_WHITESPACE_PATTERN = re.compile(r'\s+')

    def __init__(self, phone_number, country_calling_code=None):
        self.raw_phone_number = phone_number or ''
        self._country_calling_code = country_calling_code

    def get_country_calling_code(self):
        """
        Get the country code as an integer.
        """
        return self._country_calling_code

    def get_phone_number_without_country_code(self):
        """
        Get the phone number without the country code.
        """
        return self.raw_phone_number

    def can_receive_sms(self):
        return False

    def get_cleaned_phone_number(self):
        """
        Get the cleaned phone number.

        This returns `None` if we do not have (or can detect) a country calling code,
        or if the phone number is blank.

        If you just want to normalize the phone number, use :meth:`.get_normalized_phone_number`.
        """
        country_code = self.get_country_calling_code()
        phone_number = self.get_phone_number_without_country_code()
        if not country_code or not phone_number:
            return None
        return f'+{country_code}{phone_number}'

    def _strip_whitespace(self, string):
        return self.STRIP_WHITESPACE_PATTERN.sub('', string)

    def get_normalized_phone_number(self):
        """
        Get the normalized full phone number with country code (if there a country code can be detected).
        """
        cleaned_phone_number = self.get_cleaned_phone_number()
        if cleaned_phone_number:
            return cleaned_phone_number
        return self._strip_whitespace(self.raw_phone_number)


class NorwegianPhoneNumberHandler(BasePhoneNumberHandler):
    """
    Simple handler that works if you only have norwegian users.
    """
    STRIP_WHITESPACE_PATTERN = re.compile(r'\s+')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._clean_raw_phone_number = self._strip_whitespace(self.raw_phone_number)

    def get_country_calling_code(self):
        if self._country_calling_code:
            return self._country_calling_code
        if self._clean_raw_phone_number.startswith('00') and len(self._clean_raw_phone_number) == 12:
            if self._clean_raw_phone_number[2:4] == '47':
                return 47
        elif self._clean_raw_phone_number.startswith('+') and len(self._clean_raw_phone_number) == 11:
            if self._clean_raw_phone_number[1:3] == '47':
                return 47
        elif len(self._clean_raw_phone_number) == 8:
            return 47
        return None

    def get_phone_number_without_country_code(self):
        """
        Get the phone number without the country code.
        """
        if self.get_country_calling_code() != 47:
            return None
        if self._clean_raw_phone_number.startswith('00'):
            return self._clean_raw_phone_number[4:]
        elif self._clean_raw_phone_number.startswith('+'):
            return self._clean_raw_phone_number[3:]
        return self._clean_raw_phone_number

    def can_receive_sms(self):
        if self.get_country_calling_code() != 47:
            return False
        phone_number = self.get_phone_number_without_country_code()
        return phone_number.startswith('9') or phone_number.startswith('4')


def _get_handler_class():
    adapter_class_path = getattr(settings, 'IEVV_PHONE_NUMBER_HANDLER', None) \
                         or 'ievv_opensource.ievv_sms.phone_number_handler.BasePhoneNumberHandler'
    return import_string(adapter_class_path)


def get(phone_number):
    """
    Get the configured phone number handler.

    Returns:
        ievv_opensource.ievv_sms.phone_number_handler.BasePhoneNumberHandler: The currently configured
            phone number handler.
    """
    handler_class = _get_handler_class()
    return handler_class(phone_number)



import sys
from datetime import datetime


class TimeExecution(object):
    """
    Times the execution of code that is run within, and outputs with label to the

    Example:

        from ievv_opensource.utils.timer import TimeExecution
        from time import sleep
        from random import randint

        with TimeExecution(label='foo', output_function=print):
            sleep(randint(1,3))

        with TimeExecution(label='bar', output_function=logger.critical):
            sleep(randint(1,3))
    """
    def __init__(self, label, output_function=sys.stdout.write):
        self.label = label
        self.output_function = output_function
        self.start_time = None

    def __enter__(self):
        self.start_time = datetime.now()

    def __exit__(self, ttype, value, traceback):
        end_time = datetime.now()
        duration = (end_time - self.start_time).total_seconds()
        self.output_function(f'{self.label}: {duration} seconds')




class ReadOnlyAdminPreMixin(object):
    """
    Mixin for :class:`django.contrib.admin.options.ModelAdmin`
    that makes the model admin read only.

    Examples:

        Typical usage::

            from ievv_opensource.ievv_djangoadmin import ievv_djangoadmin_mixins

            @admin.register(MyModel)
            class MyModelAdmin(ievv_djangoadmin_mixins.ReadOnlyAdminPreMixin,
                               admin.ModelAdmin):
                list_display = [
                    # ...
                ]
                search_fields = [
                    # ...
                ]
                # ...

    """
    #: Overridden change form template that removes
    #: the save buttons.
    change_form_template = 'ievv_djangoadmin/ievv_djangoadmin_mixins/' \
                           'read_only_change_form.django.html'

    #: Set to ``None`` - no actions by default.
    actions = None

    def get_readonly_fields(self, request, obj=None):
        """
        Returns all the fields on the model,
        so all fields are read only.
        """
        return [f.name for f in self.model._meta.get_fields() if f.concrete]

    def get_fields(self, request, obj=None):
        return self.get_readonly_fields(request, obj=obj)

    def has_add_permission(self, request):
        """
        Returns ``False`` - so no add permission.
        """
        return False

    def has_delete_permission(self, request, id=None):
        """
        Returns ``False`` - so no delete permission.
        """
        return False



import logging

from django.core.mail.backends.base import BaseEmailBackend
from django.utils.encoding import force_text

from ievv_opensource.ievv_developemail.models import DevelopEmail


logger = logging.getLogger(__name__)


class DevelopEmailBackend(BaseEmailBackend):
    def send_messages(self, email_messages):
        developemails = []
        for email_message in email_messages:
            # message = email_message.message()
            raw_message = email_message.message().as_string()
            developemail = DevelopEmail(
                subject=email_message.subject,
                from_email=email_message.from_email,
                to_emails=', '.join(map(force_text, email_message.to)),
                raw_message=raw_message
            )

            # emailmessage = email.message_from_string(message.message().as_string())
            developemails.append(developemail)

            logger.debug(raw_message)

        DevelopEmail.objects.bulk_create(developemails)
        return len(list(email_messages))



from django.db import models


class DebugSmsMessage(models.Model):

    phone_number = models.CharField(max_length=50, db_index=True)

    message = models.TextField()

    send_as = models.CharField(max_length=50, null=False, blank=True, default='')

    created_datetime = models.DateTimeField(auto_now_add=True)



import os

import shutil
from ievv_opensource.utils.shellcommandmixin import ShellCommandError

from . import base


class NpmDocBuilder(base.AbstractDocBuilder):
    """
    Doc builder that looks for a npm script named ``build-docs``
    and runs that if it is available.

    Assumes that the ``build-docs`` task outputs its docs in ``built_docs/``
    in the source directory (the same directory as package.json).
    """
    name = 'npmdocbuilder'

    def get_script_keys(self):
        scripts_dict = self.app.get_installer('npm').get_packagejson_dict().get('scripts', {})
        return scripts_dict.keys()

    def is_available(self):
        """
        Is this docbuilder available in the app?
        """
        return 'build-docs' in self.get_script_keys()

    def __run_build_docs_npm_script(self):
        try:
            self.app.get_installer('npm').run_packagejson_script(
                script='build-docs',
                args=['--silent'])
        except ShellCommandError:
            self.get_logger().command_error('documentation build FAILED!')
            raise SystemExit()

    def __get_staticsources_build_directory(self):
        return self.app.get_source_path('built_docs')

    def __verify_docs_built_in_correct_directory(self):
        if not os.path.exists(self.__get_staticsources_build_directory()):
            self.get_logger().error(
                '"npm run build-docs" was executed successfully, '
                'but it did not output the results in {expected_build_directory!r}.'.format(
                    expected_build_directory=self.__get_staticsources_build_directory()
                ))
            self.get_logger().command_error('documentation build FAILED!')
            raise SystemExit()

    def __move_docs_into_output_directory(self, output_directory):
        if os.path.exists(output_directory):
            shutil.rmtree(output_directory)
        shutil.move(self.__get_staticsources_build_directory(), output_directory)

    def build_docs(self, output_directory):
        self.get_logger().command_start('Building docs for {appname}'.format(
            appname=self.app.appname))
        if os.path.exists(self.__get_staticsources_build_directory()):
            shutil.rmtree(self.__get_staticsources_build_directory())
        self.__run_build_docs_npm_script()
        self.__verify_docs_built_in_correct_directory()
        self.__move_docs_into_output_directory(output_directory=output_directory)
        self.get_logger().command_success('documentation build for {appname} succeeded :)'.format(
            appname=self.app.appname
        ))



from django import test

from ievv_opensource.utils import text_utils


class TestNormalizeWhitespace(test.TestCase):
    def test_spaces(self):
        self.assertEqual(text_utils.normalize_whitespace('Hello    cruel   world'),
                         'Hello cruel world')

    def test_newlines(self):
        self.assertEqual(text_utils.normalize_whitespace('Hello\ncruel\nworld'),
                         'Hello cruel world')

    def test_tabs(self):
        self.assertEqual(text_utils.normalize_whitespace('Hello\tcruel\tworld'),
                         'Hello cruel world')



class LazyString(object):
    def __init__(self, formatstring, *args, **kwargs):
        self.formatstring = formatstring
        self.args = args
        self.kwargs = kwargs

    def __str__(self):
        return self.formatstring.format(*self.args, **self.kwargs)

    def __repr__(self):
        return 'LazyString({!r}, args={!r}, kwargs={!r})'.format(
            self.formatstring,
            self.args,
            self.kwargs)




class Singleton(object):
    """
    Implements the singleton pattern.

    Example:

        Create a singleton class::

            class MySingleton(Singleton):
                def __init__(self):
                    super().__init__()
                    self.value = 0

                def add(self):
                    self.value += 1


        Use the singleton::

            MySingleton.get_instance().add()
            MySingleton.get_instance().add()
            print(MySingleton.get_instance().value)
    """
    _instance = None

    def __init__(self):
        """
        Ensures there is only one instance created. Make sure to
        use super() in subclasses.
        """
        if self._instance:
            raise ValueError('{}.{} is a singleton and can only have a single instance.'.format(
                self.__class__.__module__,
                self.__class__.__name__))

    @classmethod
    def get_instance(cls):
        """
        Get an instance of the singleton.
        """
        if not cls._instance:
            cls._instance = cls()
        return cls._instance



from django.conf import settings
from django.core.management.base import BaseCommand
from ievv_opensource.utils import ievvbuildstatic


class Command(BaseCommand):
    help = 'Show list of NPM package versions that is actually installed.'

    def add_arguments(self, parser):
        parser.add_argument('-a', '--appname', dest='appname',
                            required=False,
                            help='Optional app name.')

    def __make_npminstall_packages(self, app):
        npm = ievvbuildstatic.installers.npm.NpmInstaller(app=app)
        output = [
            'ievvbuildstatic.npminstall.Plugin(',
            '    packages={'
        ]
        for package_name in npm.get_installed_package_names():
            output.append('        {package_name!r}: {version!r},'.format(
                package_name=package_name,
                version=npm.get_package_version(package_name=package_name)
            ))
        output.extend([
            '    }',
            ')'
        ])
        return '\n'.join(output)

    def handle(self, *args, **options):
        appname = options['appname']
        if appname:
            apps = [settings.IEVVTASKS_BUILDSTATIC_APPS.get_app(appname=appname)]
        else:
            apps = settings.IEVVTASKS_BUILDSTATIC_APPS.iterapps()
        for app in apps:
            self.stdout.write('')
            self.stdout.write('*' * 70)
            self.stdout.write('APP: {}'.format(app.appname))
            self.stdout.write('*' * 70)
            self.stdout.write(self.__make_npminstall_packages(app=app))



from __future__ import print_function

import sys
import textwrap
import threading

from ievv_opensource.utils import ievv_colorize
from ievv_opensource.utils.desktopnotifications import desktopnotificationapi


class Logger(object):
    """
    Logger class used by :class:`.LogMixin`.
    """
    loggers = {}
    # messagelock = threading.Lock()
    DEBUG = 5
    STDOUT = 4
    INFO = 3
    WARNING = 2
    ERROR = 1

    @classmethod
    def get_instance(cls, name, level=None, command_error_message=None):
        """
        Get an instance of the logger by the given name.

        Parameters:
            name: The name of the logger.
        """
        if name not in cls.loggers:
            cls.loggers[name] = cls(name=name)
        cls.loggers[name].set_level(level=level)
        cls.loggers[name].set_command_error_message(command_error_message=command_error_message)
        return cls.loggers[name]

    def __init__(self, name, level=None):
        """
        Parameters:
            name: The name of the logger.
        """
        self.name = name
        self.level = level or self.DEBUG
        self._has_messagelock = False
        self._messagelocktimer = None

    def set_level(self, level):
        self.level = level or self.DEBUG

    def set_command_error_message(self, command_error_message):
        self.command_error_message = command_error_message

    def _acquire_messagelock(self):
        # if not self._has_messagelock:
        #     self.__class__.messagelock.acquire()
        #     self._has_messagelock = True
        pass

    def _release_messagelock(self):
        # self._messagelocktimer = None
        # try:
        #     self.__class__.messagelock.release()
        # except RuntimeError:
        #     pass
        # self._has_messagelock = False
        pass

    def _slowrelease_messagelock(self):
        if not self._messagelocktimer:
            self._messagelocktimer = threading.Timer(0.3, self._release_messagelock)
            self._messagelocktimer.start()

    def _queue_message(self, message=''):
        # self._messagequeue.append(message)
        self._acquire_messagelock()
        # self._flush_message_queue()
        print(message)
        sys.stdout.flush()
        self._slowrelease_messagelock()

    # def _flush_message_queue(self):
    #     for message in self._messagequeue:
    #         print(message)

    def stdout(self, line):
        """
        Use this to redirecting sys.stdout when running shell commands.
        """
        if self.level >= self.STDOUT:
            self._queue_message(line.rstrip())

    def stderr(self, line):
        """
        Use this to redirecting sys.stderr when running shell commands.
        """
        if self.level >= self.ERROR:
            self._queue_message(line.rstrip())

    def __colorize(self, message, **kwargs):
        return ievv_colorize.colorize(message, **kwargs)

    def __colorprint(self, message, **kwargs):
        self._queue_message(self.__colorize(message, **kwargs))

    def infobox(self, message):
        if self.level >= self.INFO:
            message = textwrap.fill(message, width=70)
            message = '\n{line}\n{message}\n{line}\n'.format(
                message=message,
                line='*' * 70
            )
            self.__colorprint(message, color=ievv_colorize.COLOR_BLUE)

    def info(self, message):
        """
        Log an info message.
        """
        if self.level >= self.INFO:
            self.__colorprint(message, color=ievv_colorize.COLOR_BLUE)

    def success(self, message):
        """
        Log a success message.
        """
        if self.level >= self.INFO:
            self.__colorprint(message, color=ievv_colorize.COLOR_GREEN)

    def warning(self, message):
        """
        Log a warning message.
        """
        if self.level >= self.WARNING:
            self.__colorprint(message, color=ievv_colorize.COLOR_YELLOW)

    def error(self, message):
        """
        Log a warning message.
        """
        if self.level >= self.ERROR:
            self.__colorprint(message, color=ievv_colorize.COLOR_RED)

    def debug(self, message):
        """
        Log a debug message.
        """
        if self.level >= self.DEBUG:
            self.__colorprint(message, color=ievv_colorize.COLOR_GREY)

    def command_start(self, message):
        """
        Log the start of a command. This should be used in the beginning
        of each :meth:`ievv_opensource.utils.ievvbuildstatic.pluginbase.Plugin.run`.
        """
        if self.level >= self.DEBUG:
            self._queue_message()
        if self.level >= self.INFO:
            self.__colorprint(message, color=ievv_colorize.COLOR_BLUE, bold=True)

    def __command_end(self, message, **kwargs):
        self.__colorprint(message, **kwargs)
        if self.level >= self.DEBUG:
            self._queue_message()

    def command_error(self, message):
        """
        Log failing end of a command. This should be used in
        :meth:`ievv_opensource.utils.ievvbuildstatic.pluginbase.Plugin.run`
        when the task fails.
        """
        self.__command_end(message, color=ievv_colorize.COLOR_RED, bold=True)
        if self.level < self.DEBUG and self.command_error_message:
            self.__colorprint(self.command_error_message,
                              color=ievv_colorize.COLOR_GREY, bold=True)
        desktopnotificationapi.show_message(
            title='ERROR - {}'.format(self.name),
            message=message)

    def command_success(self, message):
        """
        Log successful end of a command. This should be used in
        :meth:`ievv_opensource.utils.ievvbuildstatic.pluginbase.Plugin.run`
        when the task succeeds.
        """
        if self.level >= self.INFO:
            self.__command_end(message, color=ievv_colorize.COLOR_GREEN, bold=True)
            desktopnotificationapi.show_message(
                title='SUCCESS - {}'.format(self.name),
                message=message)


class LogMixin(object):
    """
    Mixin class that takes care of logging for all the classes
    in the ``ievvbuildstatic`` package.

    Subclasses must override :meth:`.~LogMixin.get_logger_name`,
    and use :meth:`.~LogMixin.get_logger`.
    """
    def get_logger_name(self):
        """
        Get the name of the logger.
        """
        raise NotImplementedError()

    def get_loglevel(self):
        return Logger.DEBUG

    def get_command_error_message(self):
        return None

    def get_logger(self):
        """
        Get an instance of :meth:`.Logger` with :meth:`.get_logger_name`
        as the logger name.
        """
        return Logger.get_instance(name=self.get_logger_name(), level=self.get_loglevel(),
                                   command_error_message=self.get_command_error_message())


if __name__ == '__main__':
    import time
    import random

    class DemoThread(threading.Thread):
        def __init__(self, *args, **kwargs):
            self.text = kwargs.pop('text')
            self.logger = Logger(name=self.text)
            super(DemoThread, self).__init__(*args, **kwargs)

        def run(self):
            for x in range(30):
                self.logger.info(self.text)
            time.sleep(random.randint(1, 3))

            for x in range(10):
                self.logger.debug(self.text)
            time.sleep(random.randint(1, 3))

            for x in range(10):
                self.logger.warning(self.text)

    threads = [
        DemoThread(text='Hello world'),
        DemoThread(text='A test'),
        DemoThread(text='Yo!'),
    ]
    for thread in threads:
        thread.start()

    for thread in threads:
        thread.join()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='BatchOperation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.now)),
                ('started_running_datetime', models.DateTimeField(null=True, blank=True)),
                ('finished_datetime', models.DateTimeField(null=True, blank=True)),
                ('context_object_id', models.PositiveIntegerField(null=True, blank=True)),
                ('operationtype', models.CharField(max_length=255, db_index=True, blank=True, default='')),
                ('status', models.CharField(max_length=12, default='unprocessed', choices=[('unprocessed', 'unprocessed'), ('running', 'running'), ('finished', 'finished')])),
                ('result', models.CharField(max_length=13, default='not-available', choices=[('not-available', 'not available yet (processing not finished)'), ('successful', 'successful'), ('failed', 'failed')])),
                ('input_data_json', models.TextField(default='', blank=True)),
                ('output_data_json', models.TextField(default='', blank=True)),
                ('context_content_type', models.ForeignKey(to='contenttypes.ContentType', blank=True, null=True, on_delete=models.CASCADE)),
                ('started_by', models.ForeignKey(to=settings.AUTH_USER_MODEL, blank=True, null=True, on_delete=models.CASCADE)),
            ],
        ),
    ]



from ievv_opensource.utils.ievvbuildstatic.options_mixin import OptionsMixin
from ievv_opensource.utils.logmixin import LogMixin
from ievv_opensource.utils.shellcommandmixin import ShellCommandMixin


class AbstractInstaller(LogMixin, ShellCommandMixin, OptionsMixin):
    """
    Base class for installers.

    Each installer defines most of their own API, the only thing they
    have in common is a reference to their
    :class:`ievv_opensource.utils.ievvbuildstatic.config.App`,
    a :obj:`~.AbstractInstaller.name` and the methods
    defined in :class:`ievv_opensource.utils.logmixin.LogMixin`
    and :class:`ievv_opensource.utils.ievvbuildstatic.shellcommand.ShellCommandMixin`.

    Plugins instantiate installers using
    :meth:`ievv_opensource.utils.ievvbuildstatic.config.App.get_installer`.
    """

    #: The name of the installer.
    name = None

    def __init__(self, app):
        """
        Parameters:
            app: The :class:`ievv_opensource.utils.ievvbuildstatic.config.App` where
                this installer object was instantiated using
                :meth:`ievv_opensource.utils.ievvbuildstatic.config.App.get_installer`.
        """
        self.app = app

    def get_logger_name(self):
        return '{}.{}'.format(self.app.get_logger_name(), self.name)

    def get_loglevel(self):
        return self.app.apps.loglevel

    def get_command_error_message(self):
        return self.app.apps.command_error_message

    def initialize(self):
        raise NotImplementedError()

    def add_deferred_success(self, message):
        self.app.add_deferred_success('[{}] {}'.format(self.name, message))

    def add_deferred_warning(self, message):
        self.app.add_deferred_warning('[{}] {}'.format(self.name, message))



import os

from ievv_opensource.utils.ievvbuildstatic import pluginbase
from ievv_opensource.utils.shellcommandmixin import ShellCommandMixin, ShellCommandError


class Plugin(pluginbase.Plugin, ShellCommandMixin):
    """
    LESS build plugin --- builds .less files into css, and supports watching
    for changes.

    Examples:

        Very simple example where the source file is in
        ``demoapp/staticsources/styles/theme.less``::

            IEVVTASKS_BUILDSTATIC_APPS = ievvbuildstatic.config.Apps(
                ievvbuildstatic.config.App(
                    appname='demoapp',
                    version='1.0.0',
                    plugins=[
                        ievvbuildstatic.lessbuild.Plugin(sourcefile='theme.less'),
                    ]
                )
            )

        A more complex example that builds a django-cradmin theme
        where sources are split in multiple directories, and
        the bower install directory is on the less path
        (the example also uses :class:`ievv_opensource.utils.ievvbuildstatic.bowerinstall.Plugin`)::

            IEVVTASKS_BUILDSTATIC_APPS = ievvbuildstatic.config.Apps(
                ievvbuildstatic.config.App(
                    appname='demoapp',
                    version='1.0.0',
                    plugins=[
                        ievvbuildstatic.bowerinstall.Plugin(
                            packages={
                                'bootstrap': '~3.1.1'
                            }
                        ),
                        ievvbuildstatic.lessbuild.Plugin(
                            sourcefolder='styles/cradmin_theme_demoapp',
                            sourcefile='theme.less',
                            other_sourcefolders=[
                                'styles/cradmin_base',
                                'styles/cradmin_theme_default',
                            ],
                            less_include_paths=[
                                'bower_components',
                            ]
                        )
                    ]
                )
            )
    """

    name = 'lessbuild'
    default_group = 'css'

    def __init__(self, sourcefile, sourcefolder='styles',
                 other_sourcefolders=None,
                 less_include_paths=None,
                 **kwargs):
        """
        Parameters:
            sourcefile: Main source file (the one including all other less files)
                relative to ``sourcefolder``.
            sourcefolder: The folder where ``sourcefile`` is located relative to
                the source folder of the :class:`~ievv_opensource.utils.ievvbuild.config.App`.
            less_include_paths: Less include paths as a list. Paths are relative
                to the source folder of the :class:`~ievv_opensource.utils.ievvbuild.config.App`.
            **kwargs: Kwargs for :class:`ievv_opensource.utils.ievvbuildstatic.pluginbase.Plugin`.
        """
        super(Plugin, self).__init__(**kwargs)
        self.sourcefolder = sourcefolder
        self.other_sourcefolders = other_sourcefolders
        self.less_include_paths = less_include_paths
        self.sourcefile = os.path.join(sourcefolder, sourcefile)

    def get_sourcefile_path(self):
        return self.app.get_source_path(self.sourcefile)

    def get_destinationfile_path(self):
        return self.app.get_destination_path(
            self.sourcefile, new_extension='.css')

    def get_other_sourcefolders_paths(self):
        return map(self.app.get_source_path, self.other_sourcefolders)

    def format_less_include_paths(self):
        if self.less_include_paths:
            return ':'.join(map(self.app.get_source_path, self.less_include_paths))
        else:
            return ''

    def get_less_version(self):
        return None

    def install(self):
        self.app.get_installer('npm').queue_install(
            'less', version=self.get_less_version())

    def run(self):
        self.get_logger().command_start('Building {}.'.format(self.get_sourcefile_path()))
        executable = self.app.get_installer('npm').find_executable('lessc')
        kwargs = {}
        less_include_paths = self.format_less_include_paths()
        if less_include_paths:
            kwargs['include_path'] = less_include_paths
            self.get_logger().info('Using --include-path={}'.format(less_include_paths))
        try:
            self.run_shell_command(executable,
                                   args=[
                                       self.get_sourcefile_path(),
                                       self.get_destinationfile_path()
                                   ],
                                   kwargs=kwargs)
        except ShellCommandError:
            self.get_logger().command_error('LESS build FAILED!')
        else:
            self.get_logger().command_success('LESS build successful :)')

    def get_watch_folders(self):
        """
        We only watch the folder where the less sources are located,
        so this returns the absolute path of the ``sourcefolder``.
        """
        folders = [self.app.get_source_path(self.sourcefolder)]
        if self.other_sourcefolders:
            folders.extend(self.get_other_sourcefolders_paths())
        return folders

    def get_watch_regexes(self):
        return ['^.+[.]less$']

    def __str__(self):
        return '{}({})'.format(super(Plugin, self).__str__(), self.sourcefile)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('taglabel', models.CharField(unique=True, max_length=30, verbose_name='Tag', help_text='Maximum 30 characters.')),
                ('tagtype', models.CharField(db_index=True, default='', max_length=255, blank=True)),
            ],
            options={
                'verbose_name_plural': 'Tags',
                'verbose_name': 'Tag',
            },
        ),
        migrations.CreateModel(
            name='TaggedObject',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType', on_delete=models.CASCADE)),
                ('tag', models.ForeignKey(to='ievv_tagframework.Tag', on_delete=models.CASCADE)),
            ],
            options={
                'verbose_name_plural': 'Tagged objects',
                'verbose_name': 'Tagged object',
            },
        ),
    ]



"""
An easy to use interface for logging events to the database models in this app.


By using this you can easily log and have a useful overview about what
is going on in e.g. scheduled scripts.

To use, add ievv_opensource.ievv_logging to INSTALLED_APPS. Run migrate, and for example
if you are running a management script called "the_foo_script" nightly, do:

    from ievv_opensource.ievv_logging.utils import IevvLogging

    ievvlogging = IevvLogging(__name__) # or any custom preferred name like 'the_foo_script'
    ievvlogging.begin()

    # the script does its work

    ievvlogging.finish(
        number_of_users_updated=12,
        number_of_users_anonymized=4
    )

"""
from django.utils import timezone

from .models import IevvLoggingEventBase, IevvLoggingEventItem


def getDuration(from_dt, to_dt, interval='default'):
    """
    Examples:

        from_dt = datetime(2019, 3, 5, 23, 8, 15)
        to_dt = datetime.now()

        print(getDuration(from_dt, to_dt))
        print(getDuration(from_dt, to_dt, interval='seconds'))
        print(getDuration(from_dt, to_dt, interval='minutes'))
    """
    duration = to_dt - from_dt
    duration_in_s = duration.total_seconds()

    def years():
        return divmod(duration_in_s, 31556926) # Seconds in a year=31556926.

    def days(seconds = None):
        return divmod(seconds or duration_in_s, 86400) # Seconds in a day = 86400

    def hours(seconds = None):
        return divmod(seconds or duration_in_s, 3600) # Seconds in an hour = 3600

    def minutes(seconds = None):
        if not seconds:
            return (0, 0)
        return divmod(seconds or duration_in_s, 60) # Seconds in a minute = 60

    def seconds(seconds = None):
        if interval == 'seconds':
            return (duration_in_s, 0)
        if seconds:
            return divmod(seconds, 1)
        return (0, 0)

    def totalDuration():
        y = years()
        d = days(y[1]) # Use remainder to calculate next variable
        h = hours(d[1])
        m = minutes(h[1])
        s = seconds(m[1])

        y_ = int(y[0])
        d_ = int(d[0])
        h_ = int(h[0])
        m_ = int(m[0])
        s_ = int(s[0])
        duration_string = f"{y_} years, {d_} days, {h_} hours, {m_} minutes and {s_} seconds"

        if duration_string.startswith('0 years'):
            duration_string = duration_string.replace('0 years, ', '')
            if duration_string.startswith('0 days'):
                duration_string = duration_string.replace('0 days, ', '')
        if duration_string == '0 hours, 0 minutes and 0 seconds':
            duration_string = 'Less than a second'
        return duration_string

    return {
        'minutes': int(minutes()[0]),
        'seconds': int(seconds()[0]),
        'default': totalDuration()
    }[interval]


class IevvLogging():

    def __init__(self, slug):
        self.slug = slug

    def _get_logging_base_from_slug(self):
        if IevvLoggingEventBase.objects.filter(slug=self.slug).exists():
            return IevvLoggingEventBase.objects.get(slug=self.slug)
        return IevvLoggingEventBase(slug=self.slug)

    def exception_as_dict(self, exception_instance):
        return {
            'repr': repr(exception_instance),
            'args': exception_instance.args,
        }

    def begin(self):
        self.loggingbase = self._get_logging_base_from_slug()
        start = timezone.now()
        self.loggingbase.last_started = start
        self.loggingbase.save()
        self.loggingitem = IevvLoggingEventItem(
            logging_base=self.loggingbase,
            start_datetime=start
        )
        self.loggingitem.save()

    def finish(self, error_occured=False, **kwargs):
        finish = timezone.now()
        duration_string = getDuration(self.loggingbase.last_started, finish)
        duration_seconds = getDuration(self.loggingbase.last_started, finish, 'seconds')
        # update the base model
        self.loggingbase.last_finished = finish
        self.loggingbase.time_spent = duration_string
        self.loggingbase.time_spent_in_seconds = duration_seconds
        self.loggingbase.success_run = not error_occured
        self.loggingbase.save()
        # update the item model
        self.loggingitem.time_spent = duration_string
        self.loggingitem.time_spent_in_seconds = duration_seconds
        self.loggingitem.data = kwargs
        self.loggingitem.end_datetime = finish
        self.loggingitem.success_run = not error_occured
        self.loggingitem.save()



from django.conf import settings
from django.contrib.auth import get_user_model

from django.core import management
from django.test import TestCase
from model_mommy import mommy


class TestSetAllPasswordsToTest(TestCase):
    def test_set_all_password_to_test(self):
        user1 = mommy.make(settings.AUTH_USER_MODEL)
        user1.set_password('old')
        user2 = mommy.make(settings.AUTH_USER_MODEL)
        user2.set_password('old')
        management.call_command('ievvtasks_set_all_passwords_to_test')
        self.assertTrue(
            get_user_model().objects.get(id=user1.id).check_password('test'))
        self.assertTrue(
            get_user_model().objects.get(id=user2.id).check_password('test'))



from __future__ import unicode_literals

import json
import os

import sh
from ievv_opensource.utils.ievvbuildstatic import pluginbase
from ievv_opensource.utils.ievvbuildstatic import utils
from ievv_opensource.utils.shellcommandmixin import ShellCommandMixin, ShellCommandError


class Plugin(pluginbase.Plugin, ShellCommandMixin):
    name = 'typescriptbuild'
    default_group = 'js'

    def __init__(self,
                 main_sourcefile="main.ts",
                 destinationfile=None,
                 sourcefile_include_patterns=None,
                 sourcefile_exclude_patterns=None,
                 sourcefolder=os.path.join('scripts', 'typescript'),
                 destinationfolder=os.path.join('scripts'),
                 extra_watchfolders=None,
                 with_function_wrapper=True,
                 lint=True,
                 lintconfig=None,
                 tsc_compiler_options=None,
                 tsc_exclude=None,
                 typings_global_dependencies=None,
                 register_tsconfig_as_temporaryfile=True,
                 **kwargs):
        super(Plugin, self).__init__(**kwargs)
        self.destinationfile = destinationfile
        self.main_sourcefile = main_sourcefile
        self.register_tsconfig_as_temporaryfile = register_tsconfig_as_temporaryfile
        self.sourcefiles = utils.RegexFileList(
            include_patterns=sourcefile_include_patterns or ['^.*\.ts'],
            exclude_patterns=sourcefile_exclude_patterns
        )
        self.destinationfolder = destinationfolder
        self.sourcefolder = sourcefolder
        self.extra_watchfolders = extra_watchfolders or []
        self.with_function_wrapper = with_function_wrapper
        self.lint = lint
        self.lintconfig = lintconfig or {}
        self.tsc_compiler_options = {
            "target": "es5",
            "module": "commonjs",
            "moduleResolution": "node",
            "sourceMap": True,
            "emitDecoratorMetadata": True,
            "experimentalDecorators": True,
            "removeComments": False,
            "noImplicitAny": True,
            "suppressImplicitAnyIndexErrors": True
        }
        if tsc_compiler_options:
            self.tsc_compiler_options.update(tsc_compiler_options)
        if tsc_exclude is None:
            self.tsc_exclude = ['node_modules']
        else:
            self.tsc_exclude = tsc_exclude
        self.typings_global_dependencies = typings_global_dependencies

    def get_sourcefolder_path(self):
        return self.app.get_source_path(self.sourcefolder)

    def get_main_sourcefile_path(self):
        return self.app.get_source_path(self.sourcefolder, self.main_sourcefile)

    def get_destinationfile_path(self):
        if self.destinationfile:
            return self.app.get_destination_path(self.destinationfolder, self.destinationfile)
        else:
            return self.app.get_destination_path(self.destinationfolder, self.main_sourcefile, new_extension='.js')

    def get_typescript_version(self):
        return None

    def get_typings_version(self):
        return None

    def get_tslint_version(self):
        return None

    def get_browserify_version(self):
        return None

    def install(self):
        self.app.get_installer('npm').queue_install(
            'typescript', version=self.get_typescript_version())
        self.app.get_installer('npm').queue_install(
            'typings', version=self.get_typings_version())
        if self.lint:
            self.app.get_installer('npm').queue_install(
                'tslint', version=self.get_tslint_version())
        self.app.get_installer('npm').queue_install(
            'browserify', version=self.get_browserify_version())

    def post_install(self):
        self.setup_typings()

    def get_tsc_executable(self):
        return self.app.get_installer('npm').find_executable('tsc')

    def get_tslint_executable(self):
        return self.app.get_installer('npm').find_executable('tslint')

    def get_typings_executable(self):
        return self.app.get_installer('npm').find_executable('typings')

    def get_all_sourcefiles(self, absolute_paths=False):
        return self.sourcefiles.get_files_as_list(rootfolder=self.get_sourcefolder_path(),
                                                  absolute_paths=absolute_paths)

    def lint_typescript_file(self, relative_filepath, lintconfig_path):
        executable = self.get_tslint_executable()
        sourcefile_path = os.path.join(self.get_sourcefolder_path(), relative_filepath)
        self.get_logger().debug('Typescript linting {!r}'.format(
            relative_filepath))
        try:
            self.run_shell_command(executable,
                                   args=[
                                       '-f', lintconfig_path,
                                       sourcefile_path
                                   ])
        except ShellCommandError:
            self.get_logger().error('Typescript linting of {!r} FAILED!'.format(
                relative_filepath))

    def make_lintconfig_file(self, temporary_directory):
        lintconfig_path = os.path.join(temporary_directory, 'coffeelint.json')
        try:
            coffeelint = sh.Command(self.get_tslint_executable())
            default_lintconfig_json = coffeelint('--makeconfig',
                                                 _err=self.log_shell_command_stderr)
        except sh.ErrorReturnCode:
            self.get_logger().error('Failed to create coffeelint config file.')
            raise ShellCommandError()
        else:
            lintconfig = json.loads(str(default_lintconfig_json))
            lintconfig.update(self.lintconfig)
            open(lintconfig_path, 'wb').write(json.dumps(lintconfig, indent=2).encode('utf-8'))
            return lintconfig_path

    def lint_typescript_files(self, temporary_directory):
        # lintconfig_path = self.make_lintconfig_file(temporary_directory=temporary_directory)
        # for relative_filepath in self.get_all_sourcefiles():
        #     self.lint_typescript_file(relative_filepath=relative_filepath,
        #                               lintconfig_path=lintconfig_path)
        pass

    def make_tsconfig_dict(self, output_directory):
        tsc_compiler_options = self.tsc_compiler_options.copy()
        tsc_compiler_options['outDir'] = output_directory
        output = {
            "compilerOptions": tsc_compiler_options,
            "include": self.get_all_sourcefiles(absolute_paths=True),
            "exclude": self.tsc_exclude
        }
        return output

    def make_tsconfig(self, output_directory):
        tsconfig_dict = self.make_tsconfig_dict(output_directory)
        path = self.app.get_source_path("tsconfig.json")
        open(path, 'wb').write(json.dumps(tsconfig_dict, indent=2).encode('utf-8'))
        if self.register_tsconfig_as_temporaryfile:
            self.register_temporary_file_or_directory(path)

    def run_typings_install(self):
        self.get_logger().debug("Running typings install")
        executable = self.get_typings_executable()
        for typings_install_key in self.typings_global_dependencies:
            try:
                self.run_shell_command(executable,
                                       args=['install', typings_install_key, '--save', '--global'],
                                       _cwd=self.app.get_source_path())
            except ShellCommandError:
                self.get_logger().error('typings install failed: {}'.format(typings_install_key))
                raise

    def setup_typings(self):
        if self.typings_global_dependencies is None:
            return
        self.run_typings_install()

    def compile_typescript(self):
        executable = self.get_tsc_executable()
        sourcefolder_path = self.get_sourcefolder_path()

        try:
            self.run_shell_command(executable, _cwd=self.app.get_source_path())
        except ShellCommandError:
            self.get_logger().error('TypeScript build of {!r} FAILED!'.format(sourcefolder_path))
            raise
        else:
            self.get_logger().debug('TypeScript build of {!r} successful :)'.format(sourcefolder_path))

    def get_browserify_executable(self):
        return self.app.get_installer('npm').find_executable('browserify')

    def merge_javascript_files(self, js_directory):
        self.get_logger().command_start(
            'Running browserify with {sourcefile} as input and {destinationfile} as output'.format(
                sourcefile=self.get_main_sourcefile_path(),
                destinationfile=self.get_destinationfile_path()))

        sourcefilepath = os.path.join(js_directory, self.main_sourcefile)
        sourcefilepath = os.path.splitext(sourcefilepath)[0] + '.js'

        destinationfolder = os.path.dirname(self.get_destinationfile_path())
        if not os.path.exists(destinationfolder):
            os.makedirs(destinationfolder)
        executable = self.get_browserify_executable()
        try:
            self.run_shell_command(executable,
                                   args=[
                                       sourcefilepath,
                                       '-o', self.get_destinationfile_path()
                                   ],
                                   _cwd=self.app.get_source_path())
        except ShellCommandError:
            self.get_logger().command_error('browserify build FAILED!')
            raise
        else:
            self.get_logger().command_success('browserify build succeeded :)')

    def build(self, temporary_directory):
        if self.lint:
            self.lint_typescript_files(temporary_directory=temporary_directory)
        js_directory = os.path.join(temporary_directory, 'js')
        os.mkdir(js_directory)
        self.make_tsconfig(js_directory)

        try:
            self.compile_typescript()
        except ShellCommandError:
            self.get_logger().command_error('Typescript build FAILED!')
        else:
            # javascript_source = self.merge_javascript_files(js_directory=js_directory)
            # destination_directory = os.path.dirname(self.get_destinationfile_path())
            # if not os.path.exists(destination_directory):
            #     os.makedirs(destination_directory)
            # open(self.get_destinationfile_path(), 'wb').write(
            #     javascript_source.encode('utf-8'))
            try:
                self.merge_javascript_files(js_directory)
            except ShellCommandError:
                pass
            else:
                self.get_logger().command_success(
                    'TypeScript build successful :) Output in {}'.format(
                        self.get_destinationfile_path()))

    def run(self):
        self.get_logger().command_start('TypeScript building all files in {!r} matching {}'.format(
            self.get_sourcefolder_path(),
            self.sourcefiles.prettyformat_patterns()))
        temporary_directory = self.make_temporary_build_directory()
        self.build(temporary_directory=temporary_directory)


    def get_extra_watchfolder_paths(self):
        return map(self.app.get_source_path, self.extra_watchfolders)

    def get_watch_folders(self):
        """
        We only watch the folder where the typescript sources are located,
        so this returns the absolute path of the ``sourcefolder``.
        """
        folders = [self.app.get_source_path(self.sourcefolder)]
        if self.extra_watchfolders:
            folders.extend(self.get_extra_watchfolder_paths())
        return folders

    def get_watch_regexes(self):
        return ['^.+[.]ts']

    def __str__(self):
        return '{}({})'.format(super(Plugin, self).__str__(), self.sourcefolder)



import os

from django.conf import settings
from django.core import management

from ievv_opensource.ievvtasks_common.base_command import BaseIevvTasksCommand
from ievv_opensource.ievvtasks_development.management.commands.ievvtasks_dump_db_as_sql import \
    get_dumpdata_filepath


class Command(BaseIevvTasksCommand):
    help = 'Recreate the database using django_dbdev migrate the database ' \
           'and load the json data dump created with ' \
           'ievvtasks_dump_devdb_as_sql.'

    def handle(self, *args, **options):
        management.call_command('ievvtasks_remove_sorl_cache_media')
        management.call_command('dbdev_reinit')
        dumpdatafile = get_dumpdata_filepath()
        if os.path.exists(dumpdatafile):
            self.stdout.write('Loading data from {}.'.format(dumpdatafile))
            management.call_command('dbdev_loaddump', dumpdatafile)
        self.stdout.write('Running management command: migrate.')
        management.call_command('migrate')
        post_management_commands = getattr(settings, 'IEVVTASKS_RECREATE_DEVDB_POST_MANAGEMENT_COMMANDS', [])
        self.run_management_commands(post_management_commands)



from .get_handler import get_handler_class, get_handler  # noqa
from .handler_shortcuts import i18n_reverse, transform_url_to_languagecode  # noqa
from .i18n_urlpatterns import i18n_patterns, I18nRegexURLResolver  # noqa



from ievv_opensource.utils.ievvbuildstatic import pluginbase
from ievv_opensource.utils.logmixin import Logger
from ievv_opensource.utils.shellcommandmixin import ShellCommandError
from ievv_opensource.utils.shellcommandmixin import ShellCommandMixin


class Plugin(pluginbase.Plugin, ShellCommandMixin):
    """
    Run javascript tests by running ``npm test`` if the
    ``package.json`` has a ``"test"`` entry in ``"scripts"``.

    Examples:

        Lets say you have the following in your package.json::

            {
              "scripts": {
                "test": "jest"
              }
            }

        You can then make ``jest`` run at startup (not on watch change)
        with::

            IEVVTASKS_BUILDSTATIC_APPS = ievvbuildstatic.config.Apps(
                ievvbuildstatic.config.App(
                    appname='demoapp',
                    version='1.0.0',
                    plugins=[
                        ievvbuildstatic.run_jstests.Plugin(),
                    ]
                )
            )

        If you have a NPM script named ``"test-debug"``, that will be run
        instead of ``"test"`` when loglevel is DEBUG. This means that
        if you add something like this to your package.json::

            {
              "scripts": {
                "test": "jest",
                "test-debug": "jest --debug"
              }
            }

        and run ``ievv buildstatic --debug``, ``jest --debug`` would
        be run instead of ``jest``.
    """
    name = 'run_jstests'
    default_group = 'jstest'

    def __init__(self, **kwargs):
        """

        Args:
            **kwargs: Kwargs for :class:`ievv_opensource.utils.ievvbuildstatic.pluginbase.Plugin`.
        """
        super(Plugin, self).__init__(**kwargs)

    def get_script_keys(self):
        scripts_dict = self.app.get_installer('npm').get_packagejson_dict().get('scripts', {})
        return scripts_dict.keys()

    def get_test_script_key(self):
        if self.get_loglevel() == Logger.DEBUG:
            script_keys = self.get_script_keys()
            if 'test-debug' in script_keys:
                return 'test-debug'
        return 'test'

    def log_shell_command_stderr(self, line):
        """
        Called by :meth:`.run_shell_command` each time the shell
        command outputs anything to stderr.
        """
        super(Plugin, self).log_shell_command_stdout(line=line)

    def has_tests(self):
        return self.get_test_script_key() in self.get_script_keys()

    def run(self):
        if self.has_tests():
            self.get_logger().command_start('Running tests for {appname}'.format(
                appname=self.app.appname))
            try:
                self.app.get_installer('npm').run_packagejson_script(
                    script=self.get_test_script_key(),
                    args=['--silent'])
            except ShellCommandError:
                self.get_logger().command_error('npm test FAILED!')
                raise SystemExit()
            else:
                self.get_logger().command_success('npm test succeeded :)')



from django.contrib.auth import get_user_model
from django.contrib.auth.hashers import make_password
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'Set the passwords of all users to "test". Very useful ' \
           'when loading a production dump locally for testing/debugging.'

    def handle(self, *args, **kwargs):
        user_model = get_user_model()
        user_model.objects.all().update(password=make_password('test'))



from threading import local

from django.conf import settings
from django.utils import translation
from ievv_opensource.ievv_i18n_url.base_url import BaseUrl

_active = local()


def get_default_languagecode():
    """
    Get the default language code activated within the current thread.

    I.e.: This returns the default languagecode that the
    :class:`ievv_opensource.ievv_i18n_url.middleware.LocaleMiddleware` sets
    as default.

    If this is called without using the middleware, or in management scripts
    etc. where the middleware is not applied, we fall back on ``settings.LANGUAGE_CODE``.

    Returns:
        str: The default languagecode for the current thread.
    """
    default_languagecode = getattr(_active, 'default_languagecode', None)
    return default_languagecode or settings.LANGUAGE_CODE


def set_default_languagecode(default_languagecode):
    """
    Used by :class:`ievv_opensource.ievv_i18n_url.middleware.LocaleMiddleware`
    to set the default languagecode in the current thread.

    .. warning::

        You will normally not want to use this, but it may be useful in management
        scripts along with calling :func:`.activate`.
    """
    _active.default_languagecode = default_languagecode


def get_active_languagecode():
    """
    Get the active language code activated within the current thread.

    I.e.: This returns the active languagecode that the
    :class:`ievv_opensource.ievv_i18n_url.middleware.LocaleMiddleware` sets
    as active. This may not be the same as the languagecode django.utils.translation.get_language()
    returns if the handler overrides
    :meth:`~ievv_opensource.ievv_i18n_url.handlers.abstract_handler.AbstractHandler.get_translation_to_activate_for_languagecode`.

    If this is called without using the middleware, or in management scripts
    etc. where the middleware is not applied, we fall back on ``settings.LANGUAGE_CODE``.

    Returns:
        str: The active languagecode for the current thread.
    """
    active_languagecode = getattr(_active, 'active_languagecode', None)
    return active_languagecode or settings.LANGUAGE_CODE


def set_active_languagecode(active_languagecode):
    """
    Used by :class:`ievv_opensource.ievv_i18n_url.middleware.LocaleMiddleware`
    to set the active languagecode in the current thread.

    .. warning::

        You will normally not want to use this, but it may be useful in management
        scripts along with calling :func:`.activate`.
    """
    _active.active_languagecode = active_languagecode


def get_active_language_urlpath_prefix():
    """
    Get the active URL path prefix within the current thread.

    I.e.: This returns the language url path prefix that the
    :class:`ievv_opensource.ievv_i18n_url.middleware.LocaleMiddleware`
    sets as active.

    If this is called without using the middleware, or in management scripts
    etc. where the middleware is not applied, we fall back on empty string.

    Returns:
        str: The URL path prefix for active language in the current thread.
    """
    active_language_urlpath_prefix = getattr(_active, 'active_language_urlpath_prefix', None)
    return active_language_urlpath_prefix or ''


def set_active_language_urlpath_prefix(urlpath_prefix):
    """
    Used by :class:`ievv_opensource.ievv_i18n_url.middleware.LocaleMiddleware`
    to set the active language url prefix in the current thread.

    .. warning::

        You will normally not want to use this, but it may be useful in management
        scripts along with calling :func:`.activate`.
    """
    _active.active_language_urlpath_prefix = urlpath_prefix


def get_active_base_url():
    """
    Get the BaseUrl activated within the current thread.

    I.e.: This returns BaseUrl that the
    :class:`ievv_opensource.ievv_i18n_url.middleware.LocaleMiddleware`
    sets as active.

    If this is called without using the middleware, or in management scripts
    etc. where the middleware is not applied, we fall back on ``BaseUrl()``.

    Returns:
        ievv_opensource.ievv_i18n_url.base_url.BaseUrl: The active BaseUrl.
    """
    active_base_url = getattr(_active, 'active_base_url', None)
    if active_base_url:
        return active_base_url
    return BaseUrl()


def set_active_base_url(active_base_url):
    """
    Used by :class:`ievv_opensource.ievv_i18n_url.middleware.LocaleMiddleware`
    to set the active language url prefix in the current thread.

    .. warning::

        You will normally not want to use this, but it may be useful in management
        scripts along with calling :func:`.activate`.
    """
    base_url = active_base_url
    _active.active_base_url = base_url


def activate(active_languagecode=None, default_languagecode=None, active_translation_languagecode=None,
             active_base_url=None, active_language_urlpath_prefix=None):
    """Activate a translation.

    This works much like ``django.utils.translation.activate()`` (and it calls that function),
    but it stores all of the stuff `ievv_i18n_url` needs to function in the current thread context.

    Args:
        active_languagecode (str): Language code to set as the active languagecode in the current thread. Defaults
            to settings.LANGUAGE_CODE. The only reason to call this without the active_languagecode argument is
            to reset the active language to the defaults. E.g.: Call ``activate()`` to reset to defaults from
            settings.
        default_languagecode (str): Default language code for the current thread. Defaults to ``active_languagecode``.
        active_translation_languagecode (str): Language code to set as the active translation in the current thread.
            I.e.: The languagecode we send to ``django.utils.translation.activate()``.
            Defaults to ``active_languagecode``.
        active_base_url (ievv_opensource.ievv_i18n_url.base_url.BaseUrl): The active base URL (E.g.: https://example.com/).
            Defaults to settings.IEVV_I18N_URL_FALLBACK_BASE_URL. Can be provided as a urllib.parse.ParseResult
            or as a string.
        active_language_urlpath_prefix (str): URL path prefix for the active language.
    """
    translation.activate(active_translation_languagecode or active_languagecode or settings.LANGUAGE_CODE)
    set_active_languagecode(active_languagecode)
    set_default_languagecode(default_languagecode or active_languagecode)
    set_active_base_url(active_base_url)
    set_active_language_urlpath_prefix(active_language_urlpath_prefix)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.core.management.base import BaseCommand, CommandError

from ievv_opensource.ievv_customsql import customsql_registry


class Command(BaseCommand):
    help = 'Run ievv_customsql.'

    def add_arguments(self, parser):
        parser.add_argument('--app', dest='appname',
                            required=False, default=None,
                            help='Only run for a specified Django app.'
                                 'If this is not specified, we run for all apps.')
        parser.add_argument('-i', '--initialize', dest='initialize',
                            required=False, action='store_true',
                            help='Initialize/setup.')
        parser.add_argument('-r', '--recreate-data', dest='recreate_data',
                            required=False, action='store_true',
                            help='Recreate data.')
        parser.add_argument('--clear', dest='clear',
                            required=False, action='store_true',
                            help='Clear data created by --initialize and --recreate-data.')

    def handle(self, *args, **options):
        verbose = options['verbosity'] >= 1
        appname = options['appname']
        clear = options['clear']
        initialize = options['initialize']
        recreate_data = options['recreate_data']
        if not initialize and not recreate_data and not clear:
            raise CommandError('--clear, -i or -r is required, and you can specify all.')

        registry = customsql_registry.Registry.get_instance()
        if appname:
            customsql_iterator = registry.iter_customsql_in_app(appname=appname)
        else:
            customsql_iterator = iter(registry)

        for customsql in customsql_iterator:
            if clear:
                if verbose:
                    self.stdout.write('Clearing: {}'.format(customsql))
                customsql.clear()
            if initialize:
                if verbose:
                    self.stdout.write('Initializing: {}'.format(customsql))
                customsql.initialize()
            if recreate_data:
                if verbose:
                    self.stdout.write('Recreating data for: {}'.format(customsql))
                customsql.recreate_data()



from django import test
from django.core.exceptions import ValidationError
from django.utils import timezone
from django_cradmin import datetimeutils
from model_mommy import mommy

from ievv_opensource.ievv_batchframework.models import BatchOperation
from ievv_opensource.python2_compatibility import mock


class TestBatchOperationModel(test.TestCase):
    def test_clean_status_finished_result_not_available(self):
        batchoperation = mommy.prepare('ievv_batchframework.BatchOperation',
                                       status=BatchOperation.STATUS_FINISHED,
                                       result=BatchOperation.RESULT_NOT_AVAILABLE)
        with self.assertRaisesMessage(ValidationError,
                                      'Must be "successful" or "failed" when status is "finished".'):
            batchoperation.clean()

    def test_clean_status_running_no_started_running_datetime(self):
        batchoperation = mommy.prepare('ievv_batchframework.BatchOperation',
                                       status=BatchOperation.STATUS_RUNNING,
                                       started_running_datetime=None)
        with self.assertRaisesMessage(ValidationError,
                                      'Can not be None when status is "running" or "finished".'):
            batchoperation.clean()

    def test_clean_status_finished_no_started_running_datetime(self):
        batchoperation = mommy.prepare('ievv_batchframework.BatchOperation',
                                       status=BatchOperation.STATUS_FINISHED,
                                       result=BatchOperation.RESULT_SUCCESSFUL,
                                       started_running_datetime=None)
        with self.assertRaisesMessage(ValidationError,
                                      'Can not be None when status is "running" or "finished".'):
            batchoperation.clean()

    def test_input_data_setter(self):
        batchoperation = mommy.prepare('ievv_batchframework.BatchOperation')
        batchoperation.input_data = {'hello': 'world'}
        self.assertEqual(
            '{"hello": "world"}',
            batchoperation.input_data_json)

    def test_input_data_getter_emptystring(self):
        batchoperation = mommy.prepare('ievv_batchframework.BatchOperation',
                                       input_data_json='')
        self.assertEqual(
            None,
            batchoperation.input_data)

    def test_input_data_getter_nonemptystring(self):
        batchoperation = mommy.prepare('ievv_batchframework.BatchOperation',
                                       input_data_json='{"hello": "world"}')
        self.assertEqual(
            {'hello': 'world'},
            batchoperation.input_data)

    def test_output_data_setter(self):
        batchoperation = mommy.prepare('ievv_batchframework.BatchOperation')
        batchoperation.output_data = {'hello': 'world'}
        self.assertEqual(
            '{"hello": "world"}',
            batchoperation.output_data_json)

    def test_output_data_getter_emptystring(self):
        batchoperation = mommy.prepare('ievv_batchframework.BatchOperation',
                                       output_data_json='')
        self.assertEqual(
            None,
            batchoperation.output_data)

    def test_output_data_getter_nonemptystring(self):
        batchoperation = mommy.prepare('ievv_batchframework.BatchOperation',
                                       output_data_json='{"hello": "world"}')
        self.assertEqual(
            {'hello': 'world'},
            batchoperation.output_data)

    def test_finish_successful(self):
        batchoperation = mommy.prepare('ievv_batchframework.BatchOperation',
                                       status=BatchOperation.STATUS_RUNNING,
                                       started_running_datetime=timezone.now())
        batchoperation.finish()
        self.assertEqual(BatchOperation.RESULT_SUCCESSFUL, batchoperation.result)

    def test_finish_failed(self):
        batchoperation = mommy.prepare('ievv_batchframework.BatchOperation',
                                       status=BatchOperation.STATUS_RUNNING,
                                       started_running_datetime=timezone.now())
        mocknow = datetimeutils.default_timezone_datetime(2016, 1, 1)
        with mock.patch('ievv_opensource.ievv_batchframework.models.timezone.now', lambda: mocknow):
            batchoperation.finish(failed=True)
        self.assertEqual(BatchOperation.RESULT_FAILED, batchoperation.result)
        self.assertEqual('', batchoperation.output_data_json)
        self.assertEqual(batchoperation.finished_datetime, mocknow)

    def test_finish_with_output_data(self):
        batchoperation = mommy.make('ievv_batchframework.BatchOperation',
                                    status=BatchOperation.STATUS_RUNNING,
                                    started_running_datetime=timezone.now())
        batchoperation.finish(output_data={'hello': 'world'})
        batchoperation = BatchOperation.objects.get(id=batchoperation.id)
        self.assertEqual(
            '{"hello": "world"}',
            batchoperation.output_data_json)

    def test_mark_as_running(self):
        batchoperation = mommy.prepare('ievv_batchframework.BatchOperation',
                                       status=BatchOperation.STATUS_UNPROCESSED,
                                       started_running_datetime=None)
        mocknow = datetimeutils.default_timezone_datetime(2016, 1, 1)
        with mock.patch('ievv_opensource.ievv_batchframework.models.timezone.now', lambda: mocknow):
            batchoperation.mark_as_running()
        self.assertEqual(BatchOperation.STATUS_RUNNING,
                         batchoperation.status)
        self.assertEqual(batchoperation.started_running_datetime, mocknow)


class TestBatchOperationManager(test.TestCase):
    def test_create_synchronous(self):
        batchoperation = BatchOperation.objects.create_synchronous()
        self.assertEqual(BatchOperation.STATUS_RUNNING, batchoperation.status)

    def test_create_synchronous_with_inputdata(self):
        batchoperation = BatchOperation.objects.create_synchronous(
            input_data={'hello': 'world'})
        self.assertEqual(
            '{"hello": "world"}',
            batchoperation.input_data_json)

    def test_create_asynchronous(self):
        batchoperation = BatchOperation.objects.create_asynchronous()
        self.assertEqual(BatchOperation.STATUS_UNPROCESSED, batchoperation.status)

    def test_create_asynchronous_with_inputdata(self):
        batchoperation = BatchOperation.objects.create_asynchronous(
            input_data={'hello': 'world'})
        self.assertEqual(
            '{"hello": "world"}',
            batchoperation.input_data_json)
